from typing import Union
from .services.payment_method_type import PaymentMethodTypeService
from .services.payment import PaymentService
from .services.checkout_page import CheckoutPageService
from .services.subscription import SubscriptionService
from .services.subscription_plan import SubscriptionPlanService
from .services.subscription_product import SubscriptionProductService
from .services.subscription_subscription_item import SubscriptionSubscriptionItemService
from .services.subscription_invoice import SubscriptionInvoiceService
from .services.subscription_invoice_item import SubscriptionInvoiceItemService
from .services.payment_link import PaymentLinkService
from .services.group_payment import GroupPaymentService
from .services.escrow import EscrowService
from .services.refund import RefundService
from .services.digital_wallet import DigitalWalletService
from .services.dispute import DisputeService
from .services.customer import CustomerService
from .services.payment_address import PaymentAddressService
from .services.customer_payment_method import CustomerPaymentMethodService
from .services.sku import SkuService
from .services.order import OrderService
from .services.order_return import OrderReturnService
from .services.coupon import CouponService
from .services.disburse import DisburseService
from .services.disburse_upload_documents import DisburseUploadDocumentsService
from .services.e_wallets import EWalletsService
from .services.issuing import IssuingService
from .services.payment_card_token import PaymentCardTokenService
from .services.issuing_simulations import IssuingSimulationsService
from .services.virtual_accounts import VirtualAccountsService
from .services.fx import FxService
from .services.localization_methods import LocalizationMethodsService
from .services.webhook_methods import WebhookMethodsService
from .services.verify import VerifyService
from .services.card_network_lookup_service import CardNetworkLookupServiceService
from .net.environment import Environment


class RapydSdk:
    def __init__(
        self,
        base_url: Union[Environment, str] = Environment.DEFAULT,
        timeout: int = 60000,
    ):
        """
        Initializes RapydSdk the SDK class.
        """

        self._base_url = (
            base_url.value if isinstance(base_url, Environment) else base_url
        )
        self.payment_method_type = PaymentMethodTypeService(base_url=self._base_url)
        self.payment = PaymentService(base_url=self._base_url)
        self.checkout_page = CheckoutPageService(base_url=self._base_url)
        self.subscription = SubscriptionService(base_url=self._base_url)
        self.subscription_plan = SubscriptionPlanService(base_url=self._base_url)
        self.subscription_product = SubscriptionProductService(base_url=self._base_url)
        self.subscription_subscription_item = SubscriptionSubscriptionItemService(
            base_url=self._base_url
        )
        self.subscription_invoice = SubscriptionInvoiceService(base_url=self._base_url)
        self.subscription_invoice_item = SubscriptionInvoiceItemService(
            base_url=self._base_url
        )
        self.payment_link = PaymentLinkService(base_url=self._base_url)
        self.group_payment = GroupPaymentService(base_url=self._base_url)
        self.escrow = EscrowService(base_url=self._base_url)
        self.refund = RefundService(base_url=self._base_url)
        self.digital_wallet = DigitalWalletService(base_url=self._base_url)
        self.dispute = DisputeService(base_url=self._base_url)
        self.customer = CustomerService(base_url=self._base_url)
        self.payment_address = PaymentAddressService(base_url=self._base_url)
        self.customer_payment_method = CustomerPaymentMethodService(
            base_url=self._base_url
        )
        self.sku = SkuService(base_url=self._base_url)
        self.order = OrderService(base_url=self._base_url)
        self.order_return = OrderReturnService(base_url=self._base_url)
        self.coupon = CouponService(base_url=self._base_url)
        self.disburse = DisburseService(base_url=self._base_url)
        self.disburse_upload_documents = DisburseUploadDocumentsService(
            base_url=self._base_url
        )
        self.e_wallets = EWalletsService(base_url=self._base_url)
        self.issuing = IssuingService(base_url=self._base_url)
        self.payment_card_token = PaymentCardTokenService(base_url=self._base_url)
        self.issuing_simulations = IssuingSimulationsService(base_url=self._base_url)
        self.virtual_accounts = VirtualAccountsService(base_url=self._base_url)
        self.fx = FxService(base_url=self._base_url)
        self.localization_methods = LocalizationMethodsService(base_url=self._base_url)
        self.webhook_methods = WebhookMethodsService(base_url=self._base_url)
        self.verify = VerifyService(base_url=self._base_url)
        self.card_network_lookup_service = CardNetworkLookupServiceService(
            base_url=self._base_url
        )
        self.set_timeout(timeout)

    def set_base_url(self, base_url: Union[Environment, str]):
        """
        Sets the base URL for the entire SDK.

        :param Union[Environment, str] base_url: The base URL to be set.
        :return: The SDK instance.
        """
        self._base_url = (
            base_url.value if isinstance(base_url, Environment) else base_url
        )

        self.payment_method_type.set_base_url(self._base_url)
        self.payment.set_base_url(self._base_url)
        self.checkout_page.set_base_url(self._base_url)
        self.subscription.set_base_url(self._base_url)
        self.subscription_plan.set_base_url(self._base_url)
        self.subscription_product.set_base_url(self._base_url)
        self.subscription_subscription_item.set_base_url(self._base_url)
        self.subscription_invoice.set_base_url(self._base_url)
        self.subscription_invoice_item.set_base_url(self._base_url)
        self.payment_link.set_base_url(self._base_url)
        self.group_payment.set_base_url(self._base_url)
        self.escrow.set_base_url(self._base_url)
        self.refund.set_base_url(self._base_url)
        self.digital_wallet.set_base_url(self._base_url)
        self.dispute.set_base_url(self._base_url)
        self.customer.set_base_url(self._base_url)
        self.payment_address.set_base_url(self._base_url)
        self.customer_payment_method.set_base_url(self._base_url)
        self.sku.set_base_url(self._base_url)
        self.order.set_base_url(self._base_url)
        self.order_return.set_base_url(self._base_url)
        self.coupon.set_base_url(self._base_url)
        self.disburse.set_base_url(self._base_url)
        self.disburse_upload_documents.set_base_url(self._base_url)
        self.e_wallets.set_base_url(self._base_url)
        self.issuing.set_base_url(self._base_url)
        self.payment_card_token.set_base_url(self._base_url)
        self.issuing_simulations.set_base_url(self._base_url)
        self.virtual_accounts.set_base_url(self._base_url)
        self.fx.set_base_url(self._base_url)
        self.localization_methods.set_base_url(self._base_url)
        self.webhook_methods.set_base_url(self._base_url)
        self.verify.set_base_url(self._base_url)
        self.card_network_lookup_service.set_base_url(self._base_url)

        return self

    def set_timeout(self, timeout: int):
        """
        Sets the timeout for the entire SDK.

        :param int timeout: The timeout (ms) to be set.
        :return: The SDK instance.
        """
        self.payment_method_type.set_timeout(timeout)
        self.payment.set_timeout(timeout)
        self.checkout_page.set_timeout(timeout)
        self.subscription.set_timeout(timeout)
        self.subscription_plan.set_timeout(timeout)
        self.subscription_product.set_timeout(timeout)
        self.subscription_subscription_item.set_timeout(timeout)
        self.subscription_invoice.set_timeout(timeout)
        self.subscription_invoice_item.set_timeout(timeout)
        self.payment_link.set_timeout(timeout)
        self.group_payment.set_timeout(timeout)
        self.escrow.set_timeout(timeout)
        self.refund.set_timeout(timeout)
        self.digital_wallet.set_timeout(timeout)
        self.dispute.set_timeout(timeout)
        self.customer.set_timeout(timeout)
        self.payment_address.set_timeout(timeout)
        self.customer_payment_method.set_timeout(timeout)
        self.sku.set_timeout(timeout)
        self.order.set_timeout(timeout)
        self.order_return.set_timeout(timeout)
        self.coupon.set_timeout(timeout)
        self.disburse.set_timeout(timeout)
        self.disburse_upload_documents.set_timeout(timeout)
        self.e_wallets.set_timeout(timeout)
        self.issuing.set_timeout(timeout)
        self.payment_card_token.set_timeout(timeout)
        self.issuing_simulations.set_timeout(timeout)
        self.virtual_accounts.set_timeout(timeout)
        self.fx.set_timeout(timeout)
        self.localization_methods.set_timeout(timeout)
        self.webhook_methods.set_timeout(timeout)
        self.verify.set_timeout(timeout)
        self.card_network_lookup_service.set_timeout(timeout)

        return self


# c029837e0e474b76bc487506e8799df5e3335891efe4fb02bda7a1441840310c



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardsPersonalizeBody(BaseModel):
    """CardsPersonalizeBody

    :param card: The card token, which is a string starting with **card_**. If the client is PCI-certified, you can use the actual card number.
    :type card: str
    :param ewallet_contact: ID of the wallet contact that the card is issued to. String starting with **cont_**.
    :type ewallet_contact: str
    """

    def __init__(self, card: str, ewallet_contact: str):
        """CardsPersonalizeBody

        :param card: The card token, which is a string starting with **card_**. If the client is PCI-certified, you can use the actual card number.
        :type card: str
        :param ewallet_contact: ID of the wallet contact that the card is issued to. String starting with **cont_**.
        :type ewallet_contact: str
        """
        self.card = card
        self.ewallet_contact = ewallet_contact



from typing import Union
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel


class UpToGuard(OneOfBaseModel):
    class_list = {"str": str, "float": float}


UpTo = Union[str, float]


@JsonMap({})
class PlanTiers(BaseModel):
    """PlanTiers

    :param amount: The price for each item in this tier. Decimal number., defaults to None
    :type amount: float, optional
    :param flat_amount: Additional price for the entire tier. Default is 0., defaults to None
    :type flat_amount: float, optional
    :param up_to: The upper volume limit for this tier. Valid values: inf (infinite) or an integer., defaults to None
    :type up_to: UpTo, optional
    """

    def __init__(
        self, amount: float = None, flat_amount: float = None, up_to: UpTo = None
    ):
        """PlanTiers

        :param amount: The price for each item in this tier. Decimal number., defaults to None
        :type amount: float, optional
        :param flat_amount: Additional price for the entire tier. Default is 0., defaults to None
        :type flat_amount: float, optional
        :param up_to: The upper volume limit for this tier. Valid values: inf (infinite) or an integer., defaults to None
        :type up_to: UpTo, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.flat_amount = self._define_number(
            "flat_amount", flat_amount, nullable=True
        )
        self.up_to = UpToGuard.return_one_of(up_to)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .payment import Payment


@JsonMap({"id_": "id"})
class InlineResponse200_32Data(BaseModel):
    """InlineResponse200_32Data

    :param amount: Amount of the group payment, in units defined by currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal., defaults to None
    :type amount: float, optional
    :param amount_to_replace: Indicates the total amount of failed payments that have not been replaced. Response only., defaults to None
    :type amount_to_replace: float, optional
    :param cancel_reason: Reason for cancellation of the group payment. Response only., defaults to None
    :type cancel_reason: str, optional
    :param country: Country where the payment methods are supported. Two-letter ISO 3166-1 ALPHA-2 code. Response only., defaults to None
    :type country: str, optional
    :param currency: Currency of the payments. Three-letter ISO 4217 code., defaults to None
    :type currency: str, optional
    :param description: Description of the group payment., defaults to None
    :type description: str, optional
    :param expiration: End of the time allowed for customers to make this payment, in Unix time. Response only., defaults to None
    :type expiration: float, optional
    :param id_: ID of the Group Payment object. String starting with **gp_**., defaults to None
    :type id_: str, optional
    :param merchant_reference_id: Merchant-defined ID. Limited to 255 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payments: payments, defaults to None
    :type payments: List[Payment], optional
    :param status: Indicates the status of the group payment operation, defaults to None
    :type status: str, optional
    """

    def __init__(
        self,
        amount: float = None,
        amount_to_replace: float = None,
        cancel_reason: str = None,
        country: str = None,
        currency: str = None,
        description: str = None,
        expiration: float = None,
        id_: str = None,
        merchant_reference_id: str = None,
        metadata: dict = None,
        payments: List[Payment] = None,
        status: str = None,
    ):
        """InlineResponse200_32Data

        :param amount: Amount of the group payment, in units defined by currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal., defaults to None
        :type amount: float, optional
        :param amount_to_replace: Indicates the total amount of failed payments that have not been replaced. Response only., defaults to None
        :type amount_to_replace: float, optional
        :param cancel_reason: Reason for cancellation of the group payment. Response only., defaults to None
        :type cancel_reason: str, optional
        :param country: Country where the payment methods are supported. Two-letter ISO 3166-1 ALPHA-2 code. Response only., defaults to None
        :type country: str, optional
        :param currency: Currency of the payments. Three-letter ISO 4217 code., defaults to None
        :type currency: str, optional
        :param description: Description of the group payment., defaults to None
        :type description: str, optional
        :param expiration: End of the time allowed for customers to make this payment, in Unix time. Response only., defaults to None
        :type expiration: float, optional
        :param id_: ID of the Group Payment object. String starting with **gp_**., defaults to None
        :type id_: str, optional
        :param merchant_reference_id: Merchant-defined ID. Limited to 255 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payments: payments, defaults to None
        :type payments: List[Payment], optional
        :param status: Indicates the status of the group payment operation, defaults to None
        :type status: str, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.amount_to_replace = self._define_number(
            "amount_to_replace", amount_to_replace, nullable=True
        )
        self.cancel_reason = self._define_str(
            "cancel_reason", cancel_reason, nullable=True
        )
        self.country = self._define_str("country", country, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.description = self._define_str("description", description, nullable=True)
        self.expiration = self._define_number("expiration", expiration, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.metadata = metadata
        self.payments = self._define_list(payments, Payment)
        self.status = self._define_str("status", status, nullable=True)



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class SearchArea(Enum):
    """An enumeration representing different categories.

    :cvar GLOBAL: "global"
    :vartype GLOBAL: str
    :cvar LOCAL: "local"
    :vartype LOCAL: str
    :cvar REGIONAL: "regional"
    :vartype REGIONAL: str
    """

    GLOBAL = "global"
    LOCAL = "local"
    REGIONAL = "regional"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, SearchArea._member_map_.values()))


@JsonMap({})
class V1cnlterminationQuerySearchCriteria(BaseModel):
    """Specifies search criteria for the query.

    :param search_area: The geographic range of the query. One of the following:\<BR\>* **global** - The query is geographically unlimited.\<BR\>* **local** - The query is limited to the merchant's country.\<BR\>* **regional** - The query is limited to the merchant's region, such as APAC., defaults to None
    :type search_area: SearchArea, optional
    """

    def __init__(self, search_area: SearchArea = None):
        """Specifies search criteria for the query.

        :param search_area: The geographic range of the query. One of the following:\<BR\>* **global** - The query is geographically unlimited.\<BR\>* **local** - The query is limited to the merchant's country.\<BR\>* **regional** - The query is limited to the merchant's region, such as APAC., defaults to None
        :type search_area: SearchArea, optional
        """
        self.search_area = (
            self._enum_matching(search_area, SearchArea.list(), "search_area")
            if search_area
            else None
        )



from .inline_response_200 import InlineResponse200
from .inline_response_200_1 import InlineResponse200_1
from .inline_response_200_2 import InlineResponse200_2
from .customer import Customer
from .v1_payments_body import (
    V1PaymentsBody,
    InitiationType,
    V1PaymentsBodyPaymentMethod,
)
from .inline_response_200_3 import InlineResponse200_3
from .payments_payment_id_body import PaymentsPaymentIdBody
from .payment_id_capture_body import PaymentIdCaptureBody
from .payments_complete_payment_body import PaymentsCompletePaymentBody
from .inline_response_200_4 import InlineResponse200_4
from .v1_checkout_body import V1CheckoutBody
from .inline_response_200_5 import InlineResponse200_5
from .payments_subscriptions_body import (
    PaymentsSubscriptionsBody,
    PaymentsSubscriptionsBodyPaymentMethod,
)
from .inline_response_200_6 import InlineResponse200_6
from .subscriptions_subscription_id_body import (
    SubscriptionsSubscriptionIdBody,
    SubscriptionsSubscriptionIdBodyPaymentMethod,
)
from .inline_response_200_7 import InlineResponse200_7
from .checkout_subscriptions_body import (
    CheckoutSubscriptionsBody,
    CheckoutSubscriptionsBodyPaymentMethod,
)
from .inline_response_200_8 import InlineResponse200_8
from .inline_response_200_9 import InlineResponse200_9
from .inline_response_200_20 import InlineResponse200_20
from .inline_response_200_10 import InlineResponse200_10
from .v1_plans_body import V1PlansBody
from .inline_response_200_11 import InlineResponse200_11
from .plans_plan_id_body import PlansPlanIdBody
from .inline_response_200_12 import InlineResponse200_12
from .inline_response_200_13 import InlineResponse200_13
from .v1_products_body import V1ProductsBody, V1ProductsBodyType
from .inline_response_200_14 import InlineResponse200_14
from .products_products_id_body import ProductsProductsIdBody
from .inline_response_200_15 import InlineResponse200_15
from .v1_subscription_items_body import V1SubscriptionItemsBody
from .inline_response_200_16 import InlineResponse200_16
from .subscription_items_subscription_item_id_body import (
    SubscriptionItemsSubscriptionItemIdBody,
)
from .inline_response_200_17 import InlineResponse200_17
from .inline_response_200_18 import InlineResponse200_18
from .subscription_item_id_usage_records_body import SubscriptionItemIdUsageRecordsBody
from .inline_response_200_19 import InlineResponse200_19
from .inline_response_200_21 import InlineResponse200_21
from .v1_invoices_body import V1InvoicesBody
from .inline_response_200_22 import InlineResponse200_22
from .invoices_invoice_id_body import InvoicesInvoiceIdBody
from .inline_response_200_23 import InlineResponse200_23
from .invoice_id_pay_body import InvoiceIdPayBody
from .inline_response_200_24 import InlineResponse200_24
from .v1_invoice_items_body import V1InvoiceItemsBody
from .invoice_items_invoice_item_body import InvoiceItemsInvoiceItemBody
from .inline_response_200_25 import InlineResponse200_25
from .inline_response_200_26 import InlineResponse200_26
from .collect_payments_body import CollectPaymentsBody
from .payments_group_payments_body import PaymentsGroupPaymentsBody
from .inline_response_200_27 import InlineResponse200_27
from .inline_response_200_28 import InlineResponse200_28
from .escrow_escrow_releases_body import EscrowEscrowReleasesBody
from .inline_response_200_29 import InlineResponse200_29
from .inline_response_200_30 import InlineResponse200_30
from .v1_refunds_body import V1RefundsBody
from .inline_response_200_31 import InlineResponse200_31
from .refunds_complete_body import RefundsCompleteBody
from .refunds_group_payments_body import RefundsGroupPaymentsBody
from .inline_response_200_32 import InlineResponse200_32
from .refunds_refund_id_body import RefundsRefundIdBody
from .apple_pay_object import ApplePayObject
from .inline_response_200_33 import InlineResponse200_33
from .inline_response_200_34 import InlineResponse200_34
from .get_disputes_list_by_org_id_status import GetDisputesListByOrgIdStatus
from .inline_response_200_35 import InlineResponse200_35
from .inline_response_200_36 import InlineResponse200_36
from .v1_customers_body import V1CustomersBody
from .inline_response_200_37 import InlineResponse200_37
from .customer_request import CustomerRequest
from .inline_response_200_38 import InlineResponse200_38
from .inline_response_200_39 import InlineResponse200_39
from .v1_addresses_body import V1AddressesBody
from .inline_response_200_40 import InlineResponse200_40
from .inline_response_200_41 import InlineResponse200_41
from .category import Category
from .customer_id_payment_methods_body import CustomerIdPaymentMethodsBody
from .inline_response_200_42 import InlineResponse200_42
from .customer_payment_method import (
    CustomerPaymentMethod,
    CustomerPaymentMethod1,
    CustomerPaymentMethod2,
    CustomerPaymentMethod3,
)
from .inline_response_200_43 import InlineResponse200_43
from .inline_response_200_44 import InlineResponse200_44
from .skus_sku_id_body import SkusSkuIdBody
from .inline_response_200_45 import InlineResponse200_45
from .v1_skus_body import V1SkusBody
from .inline_response_200_46 import InlineResponse200_46
from .v1_orders_body import V1OrdersBody
from .inline_response_200_47 import InlineResponse200_47
from .orders_order_id_body import OrdersOrderIdBody, OrdersOrderIdBodyStatus
from .order_id_pay_body import OrderIdPayBody
from .order_id_returns_body import OrderIdReturnsBody
from .inline_response_200_48 import InlineResponse200_48
from .inline_response_200_49 import InlineResponse200_49
from .inline_response_200_50 import InlineResponse200_50
from .coupon import Coupon, Duration
from .inline_response_200_51 import InlineResponse200_51
from .inline_response_200_52 import InlineResponse200_52
from .inline_response_200_53 import InlineResponse200_53
from .inline_response_200_54 import InlineResponse200_54
from .v1_payouts_body import (
    V1PayoutsBody,
    V1PayoutsBodyBeneficiary,
    V1PayoutsBodyBeneficiaryEntityType,
    V1PayoutsBodySender,
)
from .inline_response_200_55 import InlineResponse200_55
from .inline_response_200_56 import InlineResponse200_56
from .payouts_beneficiary_body import PayoutsBeneficiaryBody
from .inline_response_200_57 import InlineResponse200_57
from .payouts_extended_beneficiary_body import (
    PayoutsExtendedBeneficiaryBody,
    PayoutsExtendedBeneficiaryBodyIdentificationType,
)
from .beneficiary_validate_body import (
    BeneficiaryValidateBody,
    BeneficiaryValidateBodyBeneficiary,
)
from .inline_response_200_58 import InlineResponse200_58
from .beneficiary_beneficiary_id_body import (
    BeneficiaryBeneficiaryIdBody,
    BeneficiaryBeneficiaryIdBodyGender,
    BeneficiaryBeneficiaryIdBodyIdentificationType,
)
from .inline_response_200_59 import InlineResponse200_59
from .disburse_beneficiary_body import (
    DisburseBeneficiaryBody,
    DisburseBeneficiaryBodyBeneficiaryEntityType,
    DisburseBeneficiaryBodyBeneficiaryExtendedFields,
    DisburseBeneficiaryBodyCategory,
    DisburseBeneficiaryBodySenderEntityType,
)
from .inline_response_200_60 import InlineResponse200_60
from .payouts_sender_body import PayoutsSenderBody
from .inline_response_200_61 import InlineResponse200_61
from .inline_response_200_62 import InlineResponse200_62
from .inline_response_200_63 import InlineResponse200_63
from .payouts_payout_id_body import PayoutsPayoutIdBody
from .inline_response_200_64 import InlineResponse200_64
from .inline_response_200_65 import InlineResponse200_65
from .inline_response_200_66 import InlineResponse200_66
from .account_transfer_body import AccountTransferBody
from .inline_response_200_67 import InlineResponse200_67
from .transfer_response_body import TransferResponseBody
from .account_deposit_body import AccountDepositBody
from .inline_response_200_68 import InlineResponse200_68
from .account_withdraw_body import AccountWithdrawBody
from .balance_hold_body import BalanceHoldBody
from .inline_response_200_69 import InlineResponse200_69
from .inline_response_200_70 import InlineResponse200_70
from .ewallet_id_contacts_body import EwalletIdContactsBody
from .inline_response_200_71 import InlineResponse200_71
from .contact import (
    Contact,
    ContactType,
    ContactGender,
    HouseType,
    MaritalStatus,
    VerificationStatus,
)
from .inline_response_200_72 import InlineResponse200_72
from .inline_response_200_73 import InlineResponse200_73
from .inline_response_200_97 import InlineResponse200_97
from .v1_ewallets_body import V1EwalletsBody
from .inline_response_200_98 import InlineResponse200_98
from .ewallets_ewallet_token_body import EwalletsEwalletTokenBody
from .update_ewallet_status_status import UpdateEwalletStatusStatus
from .account_limits_body import AccountLimitsBody
from .inline_response_200_99 import InlineResponse200_99
from .inline_response_200_100 import InlineResponse200_100
from .inline_response_200_101 import InlineResponse200_101
from .inline_response_200_102 import InlineResponse200_102
from .inline_response_200_103 import InlineResponse200_103
from .card_details_card_token_body import CardDetailsCardTokenBody
from .inline_response_200_74 import InlineResponse200_74, InlineResponse200_74Data
from .inline_response_200_75 import InlineResponse200_75, InlineResponse200_75Data
from .issuing_cards_body import IssuingCardsBody
from .inline_response_200_76 import InlineResponse200_76, InlineResponse200_76Data
from .inline_response_200_77 import InlineResponse200_77, InlineResponse200_77Data
from .cards_activate_body import CardsActivateBody
from .inline_response_200_78 import InlineResponse200_78
from .issuing_activate_card_body import IssuingActivateCardBody
from .inline_response_200_79 import InlineResponse200_79
from .issuing_pin_body import IssuingPinBody
from .inline_response_200_80 import InlineResponse200_80
from .cards_personalize_body import CardsPersonalizeBody
from .cards_status_body import CardsStatusBody
from .inline_response_200_81 import InlineResponse200_81
from .inline_response_200_82 import InlineResponse200_82
from .cards_pin_body import CardsPinBody
from .inline_response_200_83 import InlineResponse200_83
from .card_tokens_google_pay_body import CardTokensGooglePayBody
from .inline_response_200_84 import InlineResponse200_84
from .issuing_bankaccounts_body import IssuingBankaccountsBody
from .inline_response_200_107 import InlineResponse200_107
from .bankaccounts_bankaccounttransfertobankaccount_body import (
    BankaccountsBankaccounttransfertobankaccountBody,
)
from .inline_response_200_108 import InlineResponse200_108
from .inline_response_200_109 import InlineResponse200_109
from .bankaccounts_virtual_account_id_body import BankaccountsVirtualAccountIdBody
from .inline_response_200_93 import InlineResponse200_93
from .inline_response_200_110 import InlineResponse200_110
from .collect_card_body import CollectCardBody
from .inline_response_200_85 import InlineResponse200_85
from .cards_simulate_block_body import CardsSimulateBlockBody
from .inline_response_200_86 import InlineResponse200_86
from .cards_authorization_body import CardsAuthorizationBody, FinancialImpact
from .inline_response_200_87 import InlineResponse200_87
from .cards_reversal_body import CardsReversalBody
from .inline_response_200_88 import InlineResponse200_88
from .cards_clearing_body import CardsClearingBody
from .inline_response_200_89 import InlineResponse200_89
from .cards_refund_body import CardsRefundBody
from .cards_adjustment_body import CardsAdjustmentBody
from .v1_virtual_accounts_body import V1VirtualAccountsBody
from .inline_response_200_90 import InlineResponse200_90
from .virtual_accounts_transactions_body import VirtualAccountsTransactionsBody
from .inline_response_200_91 import InlineResponse200_91
from .inline_response_200_92 import InlineResponse200_92
from .virtual_accounts_virtual_account_id_body import (
    VirtualAccountsVirtualAccountIdBody,
)
from .inline_response_200_94 import InlineResponse200_94
from .inline_response_200_95 import InlineResponse200_95
from .inline_response_200_96 import InlineResponse200_96
from .inline_response_200_104 import InlineResponse200_104
from .inline_response_200_120 import InlineResponse200_120
from .inline_response_200_121 import InlineResponse200_121
from .inline_response_200_122 import InlineResponse200_122
from .inline_response_200_105 import InlineResponse200_105
from .inline_response_200_106 import InlineResponse200_106
from .v1_identities_body import V1IdentitiesBody
from .inline_response_200_111 import InlineResponse200_111
from .inline_response_200_112 import InlineResponse200_112
from .inline_response_200_113 import InlineResponse200_113
from .inline_response_200_114 import InlineResponse200_114
from .applications_hosted_body import ApplicationsHostedBody
from .inline_response_200_115 import InlineResponse200_115
from .inline_response_200_116 import InlineResponse200_116
from .hosted_idv_body import HostedIdvBody, HostedIdvBodyRequestType
from .inline_response_200_117 import InlineResponse200_117
from .cnl_termination_query_body import CnlTerminationQueryBody
from .inline_response_200_118 import InlineResponse200_118
from .inline_response_200_119 import InlineResponse200_119
from .status_1 import Status1
from .payment_method_type import PaymentMethodType, PaymentMethodTypePaymentFlowType
from .payment_amount_range_per_currency_inner import PaymentAmountRangePerCurrencyInner
from .field_1 import Field1, Field1Type
from .field_1_conditions import Field1Conditions, Field1ConditionsThresholdValue
from .payment_method_type_required_fields import PaymentMethodTypeRequiredFields
from .payment import Payment, PaymentMethodTypeCategory
from .address_1 import Address1
from .dispute import Dispute, DisputeStatus
from .payment_ewallets import PaymentEwallets
from .payment_instructions import PaymentInstructions
from .next_action import NextAction
from .outcome import Outcome, NetworkStatus, OutcomePaymentFlowType, RiskLevel
from .fee import Fee
from .payment_refunds import PaymentRefunds
from .payment_status import PaymentStatus
from .payment_steps import PaymentSteps
from .payment_options import PaymentOptions, PaymentOptionsPaymentFlowType
from .bin_details import BinDetails
from .condition import Condition, ConditionThresholdValue
from .discount import Discount
from .customer_payment_methods import CustomerPaymentMethods
from .subscription import (
    Subscription,
    SubscriptionBillingCycleAnchor,
    SubscriptionStatus,
    SubscriptionType,
)
from .subscription_items import SubscriptionItems
from .subscription_item import SubscriptionItem
from .plan import (
    Plan,
    AggregateUsage,
    BillingScheme,
    Interval,
    PlanProduct,
    TiersMode,
    UsageType,
)
from .plan_tiers import PlanTiers, UpTo
from .plan_transform_usage import PlanTransformUsage
from .product import Product, ProductType
from .product_package_dimensions import ProductPackageDimensions
from .sku import Sku
from .sku_package_dimensions import SkuPackageDimensions
from .fx_fee import FxFee
from .transaction_fee import TransactionFee
from .client_details_object import ClientDetailsObject, ScreenColorDepth
from .checkout_page_response import CheckoutPageResponse
from .merchant_customer_support import MerchantCustomerSupport
from .hosted_page_status import HostedPageStatus
from .hosted_page_additional_response_cart_items import (
    HostedPageAdditionalResponseCartItems,
)
from .hosted_page_additional_response_custom_elements import (
    HostedPageAdditionalResponseCustomElements,
)
from .v1paymentssubscriptions_subscription_items import (
    V1paymentssubscriptionsSubscriptionItems,
)
from .inline_response_200_7_data import InlineResponse200_7Data
from .subscription_hosted_page_reponse import (
    SubscriptionHostedPageReponse,
    SubscriptionHostedPageReponseBillingCycleAnchor,
    SubscriptionHostedPageReponseStatus,
)
from .subscription_hosted_page_reponse_custom_elements import (
    SubscriptionHostedPageReponseCustomElements,
)
from .subscription_hosted_page_reponse_merchant_customer_support import (
    SubscriptionHostedPageReponseMerchantCustomerSupport,
)
from .inline_response_200_12_data import InlineResponse200_12Data
from .v1products_package_dimensions import V1productsPackageDimensions
from .inline_response_200_17_data import InlineResponse200_17Data
from .inline_response_200_18_data import InlineResponse200_18Data
from .invoice_response import (
    InvoiceResponse,
    BillingReason,
    InvoiceResponseStatus,
    InvoiceResponseType,
)
from .invoice_item_response import InvoiceItemResponse
from .payout import Payout, PayoutPayoutType
from .invoice_item_response_period import InvoiceItemResponsePeriod
from .beneficiary import Beneficiary
from .payout_ewallets import PayoutEwallets
from .payout_instructions import PayoutInstructions
from .payout_fees import PayoutFees
from .sender import Sender
from .payout_status import PayoutStatus
from .entity_type import EntityType
from .inline_response_200_23_data import InlineResponse200_23Data
from .inline_response_200_25_data import InlineResponse200_25Data
from .payment_link import PaymentLink
from .group_payment import GroupPayment
from .escrow_response import EscrowResponse
from .escrow_response_escrow_releases import EscrowResponseEscrowReleases
from .escrow_response_escrow_releases_data import EscrowResponseEscrowReleasesData
from .escrow_ewallets import EscrowEwallets
from .refund import Refund
from .refund_ewallets import RefundEwallets
from .inline_response_200_32_data import InlineResponse200_32Data
from .apple_pay_object_response import ApplePayObjectResponse
from .customer_request_payment_method import CustomerRequestPaymentMethod
from .discount_customer_response import DiscountCustomerResponse
from .address_response import AddressResponse
from .inline_response_200_43_data import InlineResponse200_43Data
from .v1skussku_id_inventory import (
    V1skusskuIdInventory,
    V1skusskuIdInventoryType,
    Value,
)
from .v1skussku_id_package_dimensions import V1skusskuIdPackageDimensions
from .order_response import OrderResponse
from .order_item_response import OrderItemResponse
from .order_returned_item_response import OrderReturnedItemResponse
from .order_response_status_transitions import OrderResponseStatusTransitions
from .v1orders_items import V1ordersItems, V1ordersItemsType
from .v1ordersorder_idreturns_items import (
    V1ordersorderIdreturnsItems,
    V1ordersorderIdreturnsItemsType,
)
from .order_returned_response import OrderReturnedResponse
from .inline_response_200_52_data import InlineResponse200_52Data
from .payout_method_type_details import PayoutMethodTypeDetails
from .status import Status
from .payout_required_fields import PayoutRequiredFields, PayoutRequiredFieldsType
from .mass_payout_response import MassPayoutResponse
from .gender import Gender
from .inline_response_200_58_data import InlineResponse200_58Data
from .inline_response_200_59_data import InlineResponse200_59Data
from .v1hosteddisbursebeneficiary_beneficiary_optional_fields import (
    V1hosteddisbursebeneficiaryBeneficiaryOptionalFields,
)
from .hosted_beneficiary_token_response import (
    HostedBeneficiaryTokenResponse,
    HostedBeneficiaryTokenResponseBeneficiaryEntityType,
    HostedBeneficiaryTokenResponseBeneficiaryExtendedFields,
    HostedBeneficiaryTokenResponseCategory,
    HostedBeneficiaryTokenResponseEntityType,
    HostedBeneficiaryTokenResponseSenderEntityType,
    HostedBeneficiaryTokenResponseStatus,
)
from .hosted_beneficiary_token_response_beneficiary_optional_fields import (
    HostedBeneficiaryTokenResponseBeneficiaryOptionalFields,
)
from .hosted_beneficiary_token_response_merchant_customer_support import (
    HostedBeneficiaryTokenResponseMerchantCustomerSupport,
)
from .inline_response_200_62_data import InlineResponse200_62Data
from .payout_method_type import PayoutMethodType
from .payout_amount_range_per_currency_inner import PayoutAmountRangePerCurrencyInner
from .payout_returned import PayoutReturned, PayoutReturnedPayoutType
from .payout_returned_status import PayoutReturnedStatus
from .inline_response_200_65_data import InlineResponse200_65Data
from .transfer import Transfer, TransferStatus
from .inline_response_200_68_data import InlineResponse200_68Data
from .put_funds_on_hold_response import (
    PutFundsOnHoldResponse,
    PutFundsOnHoldResponseDestinationBalanceType,
    PutFundsOnHoldResponseSourceBalanceType,
)
from .contact_business import ContactBusiness, ContactBusinessEntityType
from .inline_response_200_72_data import InlineResponse200_72Data
from .inline_response_200_73_data import InlineResponse200_73Data
from .inline_response_200_73_data_compliance_levels import (
    InlineResponse200_73DataComplianceLevels,
)
from .inline_response_200_73_data_elements import InlineResponse200_73DataElements
from .ewallet import Ewallet, EwalletCategory, EwalletStatus, EwalletType
from .account import Account
from .ewallet_contacts import EwalletContacts
from .limit import Limit
from .v1ewallets_contact import V1ewalletsContact
from .ewallet_transaction import (
    EwalletTransaction,
    EwalletTransactionBalanceType,
    EwalletTransactionDestinationBalanceType,
    EwalletTransactionSourceBalanceType,
)
from .ewallet_transaction_details import (
    EwalletTransactionDetails,
    EwalletTransactionDetailsBalanceType,
)
from .inline_response_200_103_data import InlineResponse200_103Data
from .inline_response_200_103_data_bank_accounts import (
    InlineResponse200_103DataBankAccounts,
    RequestedCurrency,
    InlineResponse200_103DataBankAccountsStatus,
)
from .payment_params import PaymentParams
from .card_issuing import CardIssuing
from .card_issuing_masked import CardIssuingMasked
from .hosted_page_activate_card_response import HostedPageActivateCardResponse
from .hosted_page_card_pin_response import HostedPageCardPinResponse
from .card_transaction import CardTransaction, CardTransactionPosEntryMode
from .set_pin_response import (
    SetPinResponse,
    SetPinResponseBlockedReason,
    SetPinResponseStatus,
)
from .add_cardto_google_pay_response import AddCardtoGooglePayResponse
from .add_cardto_google_pay_response_user_address import (
    AddCardtoGooglePayResponseUserAddress,
)
from .inline_response_200_107_data import InlineResponse200_107Data
from .inline_response_200_108_data import InlineResponse200_108Data
from .inline_response_200_93_data_transactions import (
    InlineResponse200_93DataTransactions,
)
from .inline_response_200_109_data import InlineResponse200_109Data
from .inline_response_200_93_data import InlineResponse200_93Data
from .inline_response_200_110_data import InlineResponse200_110Data
from .v1hostedcollectcard_card_fields import V1hostedcollectcardCardFields
from .card_token_response import CardTokenResponse
from .card_token_response_card_fields import CardTokenResponseCardFields
from .card_token_response_payment_params import CardTokenResponsePaymentParams
from .simulate_block_card_response import (
    SimulateBlockCardResponse,
    SimulateBlockCardResponseBlockedReason,
    SimulateBlockCardResponseStatus,
)
from .simulate_card_transaction_authorization_request_eea import (
    SimulateCardTransactionAuthorizationRequestEea,
    IssuingTxnType,
    SimulateCardTransactionAuthorizationRequestEeaPosEntryMode,
)
from .simulate_card_transaction_authorization_request_eea_auth_response import (
    SimulateCardTransactionAuthorizationRequestEeaAuthResponse,
)
from .simulate_card_transaction_authorization_reversal_eea import (
    SimulateCardTransactionAuthorizationReversalEea,
    SimulateCardTransactionAuthorizationReversalEeaTxnType,
)
from .simulate_clearing_card_transaction_eea_remote_auth_response import (
    SimulateClearingCardTransactionEeaRemoteAuthResponse,
)
from .simulate_clearing_card_transaction_eea import (
    SimulateClearingCardTransactionEea,
    SimulateClearingCardTransactionEeaTxnType,
)
from .inline_response_200_90_data import InlineResponse200_90Data
from .inline_response_200_91_data import InlineResponse200_91Data
from .inline_response_200_91_data_transactions import (
    InlineResponse200_91DataTransactions,
)
from .virtual_account_issuing import VirtualAccountIssuing, VirtualAccountIssuingStatus
from .virtual_account_transaction_response import VirtualAccountTransactionResponse
from .inline_response_200_94_data import InlineResponse200_94Data
from .inline_response_200_95_data import (
    InlineResponse200_95Data,
    AcceptSwift,
    AccountIdType,
    LocalBankCodeType,
    Refundable,
    RemitterDetails,
)
from .daily_rate import DailyRate
from .list_supported_languages_response import ListSupportedLanguagesResponse
from .list_supported_languages_response_languages import (
    ListSupportedLanguagesResponseLanguages,
)
from .list_countries_response import ListCountriesResponse
from .list_countries_response_languages import ListCountriesResponseLanguages
from .list_currencies_response import ListCurrenciesResponse
from .resend_webhook_response import ResendWebhookResponse
from .list_webhooks_response_attempts import ListWebhooksResponseAttempts
from .list_webhooks_response_attempts_http_response_headers import (
    ListWebhooksResponseAttemptsHttpResponseHeaders,
)
from .list_webhooks_response import ListWebhooksResponse
from .inline_response_200_111_data import InlineResponse200_111Data
from .inline_response_200_112_data import InlineResponse200_112Data
from .inline_response_200_113_data import InlineResponse200_113Data
from .entity_type_verify import EntityTypeVerify
from .inline_response_200_114_data import (
    InlineResponse200_114Data,
    InlineResponse200_114DataStatus,
)
from .verify_hosted_app_response import VerifyHostedAppResponse
from .verify_hosted_app_response_merchant_details import (
    VerifyHostedAppResponseMerchantDetails,
)
from .verify_hosted_app_response_merchant_details_merchant_customer_support import (
    VerifyHostedAppResponseMerchantDetailsMerchantCustomerSupport,
)
from .inline_response_200_116_data import InlineResponse200_116Data, ApplicationLevel
from .inline_response_200_116_data_application_type import (
    InlineResponse200_116DataApplicationType,
)
from .inline_response_200_116_data_organization_details import (
    InlineResponse200_116DataOrganizationDetails,
)
from .inline_response_200_116_data_renew_result import (
    InlineResponse200_116DataRenewResult,
)
from .inline_response_200_116_data_organization_details_merchant_customer_support import (
    InlineResponse200_116DataOrganizationDetailsMerchantCustomerSupport,
)
from .inline_response_200_117_data import (
    InlineResponse200_117Data,
    InlineResponse200_117DataRequestType,
    InlineResponse200_117DataStatus,
)
from .inline_response_200_117_data_merchant_customer_support import (
    InlineResponse200_117DataMerchantCustomerSupport,
)
from .v1cnltermination_query_search_criteria import (
    V1cnlterminationQuerySearchCriteria,
    SearchArea,
)
from .v1cnltermination_query_queried_merchant import (
    V1cnlterminationQueryQueriedMerchant,
    V1cnlterminationQueryQueriedMerchantBusinessCategory,
)
from .v1cnltermination_query_queried_merchant_address import (
    V1cnlterminationQueryQueriedMerchantAddress,
)
from .v1cnltermination_query_queried_merchant_principals import (
    V1cnlterminationQueryQueriedMerchantPrincipals,
)
from .v1cnltermination_query_queried_merchant_principals_address import (
    V1cnlterminationQueryQueriedMerchantPrincipalsAddress,
)
from .inline_response_200_118_data import (
    InlineResponse200_118Data,
    InlineResponse200_118DataStatus,
)
from .inline_response_200_119_data import (
    InlineResponse200_119Data,
    InlineResponse200_119DataStatus,
)
from .inline_response_200_119_data_match_stats import (
    InlineResponse200_119DataMatchStats,
)
from .inline_response_200_119_data_matches import (
    InlineResponse200_119DataMatches,
    CardNetwork,
)
from .inline_response_200_119_data_query_info import InlineResponse200_119DataQueryInfo
from .inline_response_200_119_data_principals import InlineResponse200_119DataPrincipals
from .inline_response_200_119_data_matched_merchant import (
    InlineResponse200_119DataMatchedMerchant,
    InlineResponse200_119DataMatchedMerchantBusinessCategory,
)
from .inline_response_200_119_data_registration_info import (
    InlineResponse200_119DataRegistrationInfo,
)
from .inline_response_200_119_data_query_info_queried_merchant import (
    InlineResponse200_119DataQueryInfoQueriedMerchant,
    InlineResponse200_119DataQueryInfoQueriedMerchantBusinessCategory,
)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .status import Status
from .daily_rate import DailyRate


@JsonMap({})
class InlineResponse200_104(BaseModel):
    """InlineResponse200_104

    :param status: status, defaults to None
    :type status: Status, optional
    :param data: Describes currency conversion for payments and payouts. Rapyd uses a snapshot of daily foreign exchange rates fetched at 9 PM UTC. The rate returned includes the FX markup fees., defaults to None
    :type data: DailyRate, optional
    """

    def __init__(self, status: Status = None, data: DailyRate = None):
        """InlineResponse200_104

        :param status: status, defaults to None
        :type status: Status, optional
        :param data: Describes currency conversion for payments and payouts. Rapyd uses a snapshot of daily foreign exchange rates fetched at 9 PM UTC. The rate returned includes the FX markup fees., defaults to None
        :type data: DailyRate, optional
        """
        self.status = self._define_object(status, Status)
        self.data = self._define_object(data, DailyRate)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardTokensGooglePayBody(BaseModel):
    """CardTokensGooglePayBody

    :param client_device_id: Stable device identification set by the wallet provider. Could be a computer identifier or an ID tied to hardware such as TEE_ID or SE_ID.
    :type client_device_id: str
    :param client_wallet_provider: The token requester’s Id (TRID).
    :type client_wallet_provider: str
    :param client_wallet_account_id: Client provided consumer ID that identifies the wallet account holder entity.
    :type client_wallet_account_id: str
    """

    def __init__(
        self,
        client_device_id: str,
        client_wallet_provider: str,
        client_wallet_account_id: str,
    ):
        """CardTokensGooglePayBody

        :param client_device_id: Stable device identification set by the wallet provider. Could be a computer identifier or an ID tied to hardware such as TEE_ID or SE_ID.
        :type client_device_id: str
        :param client_wallet_provider: The token requester’s Id (TRID).
        :type client_wallet_provider: str
        :param client_wallet_account_id: Client provided consumer ID that identifies the wallet account holder entity.
        :type client_wallet_account_id: str
        """
        self.client_device_id = client_device_id
        self.client_wallet_provider = client_wallet_provider
        self.client_wallet_account_id = client_wallet_account_id



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardsPinBody(BaseModel):
    """CardsPinBody

    :param card: Card number or card ID.
    :type card: str
    :param new_pin: PIN code. Numeric string.
    :type new_pin: str
    """

    def __init__(self, card: str, new_pin: str):
        """CardsPinBody

        :param card: Card number or card ID.
        :type card: str
        :param new_pin: PIN code. Numeric string.
        :type new_pin: str
        """
        self.card = card
        self.new_pin = new_pin



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .merchant_customer_support import MerchantCustomerSupport


@JsonMap({"id_": "id"})
class PaymentLink(BaseModel):
    """Retrieves details of a payment link.

    :param amount: The amount of the payment, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. On the hosted payment page, the customer: \<BR\>\<BR\> Cannot modify the amount when amount_is_editable is false or not used. \<BR\>\<BR\> Can modify the amount when amount_is_editable is true and amount has a positive value. \<BR\>\<BR\> Must enter an amount when amount_is_editable is true and amount is 0, null, or not used., defaults to None
    :type amount: float, optional
    :param amount_is_editable: Determines whether the customer can edit the amount.\<br\>\<br\> **true** - The customer can edit the amount. \<br\>\<br\> **false** - The customer cannot edit the amount., defaults to None
    :type amount_is_editable: bool, optional
    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
    :type country: str, optional
    :param currency: The two-letter ISO 3166-1 ALPHA-2 code for the country. \<BR\>\<BR\> **Transactions without FX** - Indicates the currency of the amount received by the recipient. BR\>\<BR\> **FX transactions** - Indicates the currency of the amount paid by the buyer., defaults to None
    :type currency: str, optional
    :param customer: ID of a specific customer. String starting with **cus_**. Restricts the payment link to the customer., defaults to None
    :type customer: str, optional
    :param fixed_side: Indicates whether the FX rate is fixed for the buy side (seller) or for the sell side (buyer). \<BR\>\<BR\> * buy - The currency that the Rapyd Wallet receives for goods or services. Fixed side buy relates to the seller (merchant) funds. For example, a US-based merchant wants to charge 100 USD. The buyer (customer) pays the amount in MXN that converts to 100 USD. This is the default. \<BR\>\<BR\> * sell - The currency that the buyer is charged for purchasing goods or services. Fixed side sell relates to the buyer (customer) funds. For example, a US-based merchant wants to charge a buyer 2,000 MXN and will accept whatever amount in USD that is converted from 2,000 MXN., defaults to None
    :type fixed_side: str, optional
    :param id_: ID of the payment link. String starting with **hp_reuse_**., defaults to None
    :type id_: str, optional
    :param language: Determines the default language of the hosted page. For a list of values, see https://docs.rapyd.net/en/list-supported-languages.html. \<BR\>\<BR\> When this parameter is null, the language of the user's browser is used. BR\>\<BR\> If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param max_payments: Indicates the maximum number of times that the payment link can be used for payments. When not defined, there is no limit., defaults to None
    :type max_payments: float, optional
    :param merchant_alias: Client's name., defaults to None
    :type merchant_alias: str, optional
    :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal. See https://docs.rapyd.net/en/customizing-your-hosted-page.html., defaults to None
    :type merchant_color: str, optional
    :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
    :type merchant_customer_support: MerchantCustomerSupport, optional
    :param merchant_logo: URL for the image of the client's logo. To configure this field, use the Client Portal., defaults to None
    :type merchant_logo: str, optional
    :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers. To configure this field, use the Client Portal., defaults to None
    :type merchant_privacy_policy: str, optional
    :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when one or both of the following fields is unset: \<BR\>\<BR\> * `cancel_url` \<BR\>\<BR\> * `complete_url`\<BR\>\<BR\> To configure this field, use the Client Portal, defaults to None
    :type merchant_website: str, optional
    :param page_expiration: End of the time when the customer can use the payment link, in Unix time. One year after creation. \<BR\> \<BR\> **Note:** Each hosted page that the payment link generates expires 14 days after creation., defaults to None
    :type page_expiration: float, optional
    :param requested_currency: Currency for one side of an FX transaction. Three-letter ISO 4217 code. \<BR\>\<BR\>* When `fixed_side` is **sell**, it is the currency received in the Rapyd Wallet. \<BR\>\<BR\>* When `fixed_side` is **buy**, it is the currency charged to the buyer (customer)., defaults to None
    :type requested_currency: str, optional
    :param redirect_url: The payment link URL., defaults to None
    :type redirect_url: str, optional
    :param status: Status of the hosted payment page. One of the following: \<BR\>\<BR\>* **NEW** - The hosted page was created. \<BR\>\<BR\>* **EXP** - The hosted page expired., defaults to None
    :type status: str, optional
    :param template: Optional parameters for the checkout page., defaults to None
    :type template: dict, optional
    """

    def __init__(
        self,
        amount: float = None,
        amount_is_editable: bool = None,
        country: str = None,
        currency: str = None,
        customer: str = None,
        fixed_side: str = None,
        id_: str = None,
        language: str = None,
        max_payments: float = None,
        merchant_alias: str = None,
        merchant_color: str = None,
        merchant_customer_support: MerchantCustomerSupport = None,
        merchant_logo: str = None,
        merchant_privacy_policy: str = None,
        merchant_reference_id: str = None,
        merchant_website: str = None,
        page_expiration: float = None,
        requested_currency: str = None,
        redirect_url: str = None,
        status: str = None,
        template: dict = None,
    ):
        """Retrieves details of a payment link.

        :param amount: The amount of the payment, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. On the hosted payment page, the customer: \<BR\>\<BR\> Cannot modify the amount when amount_is_editable is false or not used. \<BR\>\<BR\> Can modify the amount when amount_is_editable is true and amount has a positive value. \<BR\>\<BR\> Must enter an amount when amount_is_editable is true and amount is 0, null, or not used., defaults to None
        :type amount: float, optional
        :param amount_is_editable: Determines whether the customer can edit the amount.\<br\>\<br\> **true** - The customer can edit the amount. \<br\>\<br\> **false** - The customer cannot edit the amount., defaults to None
        :type amount_is_editable: bool, optional
        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
        :type country: str, optional
        :param currency: The two-letter ISO 3166-1 ALPHA-2 code for the country. \<BR\>\<BR\> **Transactions without FX** - Indicates the currency of the amount received by the recipient. BR\>\<BR\> **FX transactions** - Indicates the currency of the amount paid by the buyer., defaults to None
        :type currency: str, optional
        :param customer: ID of a specific customer. String starting with **cus_**. Restricts the payment link to the customer., defaults to None
        :type customer: str, optional
        :param fixed_side: Indicates whether the FX rate is fixed for the buy side (seller) or for the sell side (buyer). \<BR\>\<BR\> * buy - The currency that the Rapyd Wallet receives for goods or services. Fixed side buy relates to the seller (merchant) funds. For example, a US-based merchant wants to charge 100 USD. The buyer (customer) pays the amount in MXN that converts to 100 USD. This is the default. \<BR\>\<BR\> * sell - The currency that the buyer is charged for purchasing goods or services. Fixed side sell relates to the buyer (customer) funds. For example, a US-based merchant wants to charge a buyer 2,000 MXN and will accept whatever amount in USD that is converted from 2,000 MXN., defaults to None
        :type fixed_side: str, optional
        :param id_: ID of the payment link. String starting with **hp_reuse_**., defaults to None
        :type id_: str, optional
        :param language: Determines the default language of the hosted page. For a list of values, see https://docs.rapyd.net/en/list-supported-languages.html. \<BR\>\<BR\> When this parameter is null, the language of the user's browser is used. BR\>\<BR\> If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param max_payments: Indicates the maximum number of times that the payment link can be used for payments. When not defined, there is no limit., defaults to None
        :type max_payments: float, optional
        :param merchant_alias: Client's name., defaults to None
        :type merchant_alias: str, optional
        :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal. See https://docs.rapyd.net/en/customizing-your-hosted-page.html., defaults to None
        :type merchant_color: str, optional
        :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
        :type merchant_customer_support: MerchantCustomerSupport, optional
        :param merchant_logo: URL for the image of the client's logo. To configure this field, use the Client Portal., defaults to None
        :type merchant_logo: str, optional
        :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers. To configure this field, use the Client Portal., defaults to None
        :type merchant_privacy_policy: str, optional
        :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when one or both of the following fields is unset: \<BR\>\<BR\> * `cancel_url` \<BR\>\<BR\> * `complete_url`\<BR\>\<BR\> To configure this field, use the Client Portal, defaults to None
        :type merchant_website: str, optional
        :param page_expiration: End of the time when the customer can use the payment link, in Unix time. One year after creation. \<BR\> \<BR\> **Note:** Each hosted page that the payment link generates expires 14 days after creation., defaults to None
        :type page_expiration: float, optional
        :param requested_currency: Currency for one side of an FX transaction. Three-letter ISO 4217 code. \<BR\>\<BR\>* When `fixed_side` is **sell**, it is the currency received in the Rapyd Wallet. \<BR\>\<BR\>* When `fixed_side` is **buy**, it is the currency charged to the buyer (customer)., defaults to None
        :type requested_currency: str, optional
        :param redirect_url: The payment link URL., defaults to None
        :type redirect_url: str, optional
        :param status: Status of the hosted payment page. One of the following: \<BR\>\<BR\>* **NEW** - The hosted page was created. \<BR\>\<BR\>* **EXP** - The hosted page expired., defaults to None
        :type status: str, optional
        :param template: Optional parameters for the checkout page., defaults to None
        :type template: dict, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.amount_is_editable = amount_is_editable
        self.country = self._define_str("country", country, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.customer = self._define_str("customer", customer, nullable=True)
        self.fixed_side = self._define_str("fixed_side", fixed_side, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.language = self._define_str("language", language, nullable=True)
        self.max_payments = self._define_number(
            "max_payments", max_payments, nullable=True
        )
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_customer_support = self._define_object(
            merchant_customer_support, MerchantCustomerSupport
        )
        self.merchant_logo = self._define_str(
            "merchant_logo", merchant_logo, nullable=True
        )
        self.merchant_privacy_policy = self._define_str(
            "merchant_privacy_policy", merchant_privacy_policy, nullable=True
        )
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.requested_currency = self._define_str(
            "requested_currency", requested_currency, nullable=True
        )
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.status = self._define_str("status", status, nullable=True)
        self.template = template



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .escrow_response_escrow_releases import EscrowResponseEscrowReleases


@JsonMap({"id_": "id"})
class EscrowResponse(BaseModel):
    """EscrowResponse

    :param amount_on_hold: Total amount of funds that are currently held in the escrow, in the currency defined in `currency_code` in the payment., defaults to None
    :type amount_on_hold: float, optional
    :param created_at: Date and time the escrow was created, in Unix time., defaults to None
    :type created_at: float, optional
    :param currency: The currency of the escrow balance. Three-letter ISO 4217 code., defaults to None
    :type currency: float, optional
    :param escrow_releases: Array of objects that describe individual releases., defaults to None
    :type escrow_releases: EscrowResponseEscrowReleases, optional
    :param id_: Indicates that the number of escrow releases is greater than the number returned in the response., defaults to None
    :type id_: str, optional
    :param last_payment_completion: Date and time of the completion of the last payment or partial payment, in Unix time., defaults to None
    :type last_payment_completion: float, optional
    :param payment: ID of the payment, a string starting with **payment_**., defaults to None
    :type payment: float, optional
    :param status: Status of the escrow. One of the following:\<BR\> * **pending** - The payment and escrow were created, but the payment is not completed and the funds are not in the escrow.\<BR\> * **on_hold** - The payment is completed and the funds are in escrow. canceled - The escrow is canceled.\<BR\>* **released** - All or part of the funds have been released to the wallets., defaults to None
    :type status: str, optional
    :param total_amount_released: Total amount of funds that were released to the wallets, in the currency defined in currency_code in the payment response., defaults to None
    :type total_amount_released: str, optional
    :param updated_at: Date and time of the last update to the escrow, in Unix time., defaults to None
    :type updated_at: float, optional
    """

    def __init__(
        self,
        amount_on_hold: float = None,
        created_at: float = None,
        currency: float = None,
        escrow_releases: EscrowResponseEscrowReleases = None,
        id_: str = None,
        last_payment_completion: float = None,
        payment: float = None,
        status: str = None,
        total_amount_released: str = None,
        updated_at: float = None,
    ):
        """EscrowResponse

        :param amount_on_hold: Total amount of funds that are currently held in the escrow, in the currency defined in `currency_code` in the payment., defaults to None
        :type amount_on_hold: float, optional
        :param created_at: Date and time the escrow was created, in Unix time., defaults to None
        :type created_at: float, optional
        :param currency: The currency of the escrow balance. Three-letter ISO 4217 code., defaults to None
        :type currency: float, optional
        :param escrow_releases: Array of objects that describe individual releases., defaults to None
        :type escrow_releases: EscrowResponseEscrowReleases, optional
        :param id_: Indicates that the number of escrow releases is greater than the number returned in the response., defaults to None
        :type id_: str, optional
        :param last_payment_completion: Date and time of the completion of the last payment or partial payment, in Unix time., defaults to None
        :type last_payment_completion: float, optional
        :param payment: ID of the payment, a string starting with **payment_**., defaults to None
        :type payment: float, optional
        :param status: Status of the escrow. One of the following:\<BR\> * **pending** - The payment and escrow were created, but the payment is not completed and the funds are not in the escrow.\<BR\> * **on_hold** - The payment is completed and the funds are in escrow. canceled - The escrow is canceled.\<BR\>* **released** - All or part of the funds have been released to the wallets., defaults to None
        :type status: str, optional
        :param total_amount_released: Total amount of funds that were released to the wallets, in the currency defined in currency_code in the payment response., defaults to None
        :type total_amount_released: str, optional
        :param updated_at: Date and time of the last update to the escrow, in Unix time., defaults to None
        :type updated_at: float, optional
        """
        self.amount_on_hold = self._define_number(
            "amount_on_hold", amount_on_hold, nullable=True
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency = self._define_number("currency", currency, nullable=True)
        self.escrow_releases = self._define_object(
            escrow_releases, EscrowResponseEscrowReleases
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.last_payment_completion = self._define_number(
            "last_payment_completion", last_payment_completion, nullable=True
        )
        self.payment = self._define_number("payment", payment, nullable=True)
        self.status = self._define_str("status", status, nullable=True)
        self.total_amount_released = self._define_str(
            "total_amount_released", total_amount_released, nullable=True
        )
        self.updated_at = self._define_number("updated_at", updated_at, nullable=True)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1orders_items import V1ordersItems
from .address_1 import Address1


@JsonMap({})
class V1OrdersBody(BaseModel):
    """V1OrdersBody

    :param coupon: ID of a coupon that is applied against this order. String starting with coupon_. The duration field of the coupon must be set to **repeating**, and the `duration_in_months` and `discount_duration_in_uses` fields must be set to **1**., defaults to None
    :type coupon: str, optional
    :param customer: ID of the customer. String starting with cus_.
    :type customer: str
    :param currency: Three-letter ISO 4217 code for the currency used in the objects in the 'items' list.
    :type currency: str
    :param email: Email address of the customer., defaults to None
    :type email: str, optional
    :param items: Array of line items.
    :type items: List[V1ordersItems]
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param shipping_address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
    :type shipping_address: Address1, optional
    :param tax_percent: Percentage of tax to charge. Decimal., defaults to None
    :type tax_percent: float, optional
    :param upstream_id: Merchant-defined ID for the order. If the merchant does not define an ID, Rapyd generates it., defaults to None
    :type upstream_id: str, optional
    """

    def __init__(
        self,
        customer: str,
        currency: str,
        items: List[V1ordersItems],
        coupon: str = None,
        email: str = None,
        metadata: dict = None,
        shipping_address: Address1 = None,
        tax_percent: float = None,
        upstream_id: str = None,
    ):
        """V1OrdersBody

        :param coupon: ID of a coupon that is applied against this order. String starting with coupon_. The duration field of the coupon must be set to **repeating**, and the `duration_in_months` and `discount_duration_in_uses` fields must be set to **1**., defaults to None
        :type coupon: str, optional
        :param customer: ID of the customer. String starting with cus_.
        :type customer: str
        :param currency: Three-letter ISO 4217 code for the currency used in the objects in the 'items' list.
        :type currency: str
        :param email: Email address of the customer., defaults to None
        :type email: str, optional
        :param items: Array of line items.
        :type items: List[V1ordersItems]
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param shipping_address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
        :type shipping_address: Address1, optional
        :param tax_percent: Percentage of tax to charge. Decimal., defaults to None
        :type tax_percent: float, optional
        :param upstream_id: Merchant-defined ID for the order. If the merchant does not define an ID, Rapyd generates it., defaults to None
        :type upstream_id: str, optional
        """
        self.coupon = self._define_str("coupon", coupon, nullable=True)
        self.customer = customer
        self.currency = currency
        self.email = self._define_str("email", email, nullable=True)
        self.items = self._define_list(items, V1ordersItems)
        self.metadata = metadata
        self.shipping_address = self._define_object(shipping_address, Address1)
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )
        self.upstream_id = self._define_str("upstream_id", upstream_id, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class ApplePayObject(BaseModel):
    """ApplePayObject

    :param display_name: The canonical name for the client's business, suitable for display. 64 or fewer UTF-8 characters. Relevant to Apple Pay.
    :type display_name: str
    :param initiative_context: The client's fully qualified domain name, without the final period. For example, **acmegeneralproducts.rapyd.net**
    :type initiative_context: str
    """

    def __init__(self, display_name: str, initiative_context: str):
        """ApplePayObject

        :param display_name: The canonical name for the client's business, suitable for display. 64 or fewer UTF-8 characters. Relevant to Apple Pay.
        :type display_name: str
        :param initiative_context: The client's fully qualified domain name, without the final period. For example, **acmegeneralproducts.rapyd.net**
        :type initiative_context: str
        """
        self.display_name = display_name
        self.initiative_context = initiative_context



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .payout_amount_range_per_currency_inner import PayoutAmountRangePerCurrencyInner
from .entity_type import EntityType
from .category import Category


@JsonMap({})
class PayoutMethodType(BaseModel):
    """PayoutMethodType

    :param amount_range_per_currency: An array of objects that describe limits on the amount, per currency. Contains the following fields: maximum_amount - Maximum amount supported by this payout method for the indicated currency. Decimal number. minimum_amount - Minimum amount supported by this payout method for the indicated currency. Decimal number. * payout_currency - Currency of the payout. Three-letter ISO 4217 code. Uppercase., defaults to None
    :type amount_range_per_currency: List[List[PayoutAmountRangePerCurrencyInner]], optional
    :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type beneficiary_country: str, optional
    :param beneficiary_entity_types: A list of the beneficiary entity types supported by this payout method. One or more of the following: company individual Response only., defaults to None
    :type beneficiary_entity_types: List[EntityType], optional
    :param category: category, defaults to None
    :type category: Category, optional
    :param image: URL of an image that the merchant can use to represent the payout method., defaults to None
    :type image: str, optional
    :param is_cancelable: Indicates whether the payout can be canceled. Relevant when category is cash. One of the following values 0 - Not cancelable. 1 - cancelable.', defaults to None
    :type is_cancelable: int, optional
    :param is_expirable: Indicates whether the payout expires if not completed. Relevant when category is cash. One of the following values 0 - Not expirable. 1 - expirable.', defaults to None
    :type is_expirable: int, optional
    :param is_location_specific: Indicates whether the payout must be made at a specific physical location. Relevant when category is cash. One of the following values 0 - Not locationspecific. 1 - location specific.', defaults to None
    :type is_location_specific: int, optional
    :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payout. Relevant when is_expirable is true., defaults to None
    :type maximum_expiration_seconds: float, optional
    :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payout. Relevant when is_expirable is true., defaults to None
    :type minimum_expiration_seconds: float, optional
    :param name: Payout method name., defaults to None
    :type name: str, optional
    :param payout_currencies: A list of the currencies supported by this payout method. Three-letter ISO 4217 code. Uppercase. Response only., defaults to None
    :type payout_currencies: List[str], optional
    :param payout_method_type: The type of payout method. The two-letter prefix must match the beneficiary country code. Required when default_payout_method_type is not used., defaults to None
    :type payout_method_type: str, optional
    :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
    :type sender_country: str, optional
    :param sender_currencies: List of currencies that the sender is paying with. Currency consists of a three-letter ISO 4217 code. Response only., defaults to None
    :type sender_currencies: List[str], optional
    :param sender_entity_types: A list of the sender entity types supported by this payout method. One or more of the following: company individual Response only. , defaults to None
    :type sender_entity_types: List[EntityType], optional
    :param estimated_time_of_arrival: The estimated time period in which the beneficiary will receive the funds., defaults to None
    :type estimated_time_of_arrival: str, optional
    :param status: Indicates whether the payout method is currently available. One of the following values: 0 - Not available. 1 - Available., defaults to None
    :type status: int, optional
    """

    def __init__(
        self,
        amount_range_per_currency: List[List[PayoutAmountRangePerCurrencyInner]] = None,
        beneficiary_country: str = None,
        beneficiary_entity_types: List[EntityType] = None,
        category: Category = None,
        image: str = None,
        is_cancelable: int = None,
        is_expirable: int = None,
        is_location_specific: int = None,
        maximum_expiration_seconds: float = None,
        minimum_expiration_seconds: float = None,
        name: str = None,
        payout_currencies: List[str] = None,
        payout_method_type: str = None,
        sender_country: str = None,
        sender_currencies: List[str] = None,
        sender_entity_types: List[EntityType] = None,
        estimated_time_of_arrival: str = None,
        status: int = None,
    ):
        """PayoutMethodType

        :param amount_range_per_currency: An array of objects that describe limits on the amount, per currency. Contains the following fields: maximum_amount - Maximum amount supported by this payout method for the indicated currency. Decimal number. minimum_amount - Minimum amount supported by this payout method for the indicated currency. Decimal number. * payout_currency - Currency of the payout. Three-letter ISO 4217 code. Uppercase., defaults to None
        :type amount_range_per_currency: List[List[PayoutAmountRangePerCurrencyInner]], optional
        :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type beneficiary_country: str, optional
        :param beneficiary_entity_types: A list of the beneficiary entity types supported by this payout method. One or more of the following: company individual Response only., defaults to None
        :type beneficiary_entity_types: List[EntityType], optional
        :param category: category, defaults to None
        :type category: Category, optional
        :param image: URL of an image that the merchant can use to represent the payout method., defaults to None
        :type image: str, optional
        :param is_cancelable: Indicates whether the payout can be canceled. Relevant when category is cash. One of the following values 0 - Not cancelable. 1 - cancelable.', defaults to None
        :type is_cancelable: int, optional
        :param is_expirable: Indicates whether the payout expires if not completed. Relevant when category is cash. One of the following values 0 - Not expirable. 1 - expirable.', defaults to None
        :type is_expirable: int, optional
        :param is_location_specific: Indicates whether the payout must be made at a specific physical location. Relevant when category is cash. One of the following values 0 - Not locationspecific. 1 - location specific.', defaults to None
        :type is_location_specific: int, optional
        :param maximum_expiration_seconds: The maximum time (in seconds) that the merchant can set for completing the payout. Relevant when is_expirable is true., defaults to None
        :type maximum_expiration_seconds: float, optional
        :param minimum_expiration_seconds: The minimum time (in seconds) that the merchant can set for completing the payout. Relevant when is_expirable is true., defaults to None
        :type minimum_expiration_seconds: float, optional
        :param name: Payout method name., defaults to None
        :type name: str, optional
        :param payout_currencies: A list of the currencies supported by this payout method. Three-letter ISO 4217 code. Uppercase. Response only., defaults to None
        :type payout_currencies: List[str], optional
        :param payout_method_type: The type of payout method. The two-letter prefix must match the beneficiary country code. Required when default_payout_method_type is not used., defaults to None
        :type payout_method_type: str, optional
        :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code., defaults to None
        :type sender_country: str, optional
        :param sender_currencies: List of currencies that the sender is paying with. Currency consists of a three-letter ISO 4217 code. Response only., defaults to None
        :type sender_currencies: List[str], optional
        :param sender_entity_types: A list of the sender entity types supported by this payout method. One or more of the following: company individual Response only. , defaults to None
        :type sender_entity_types: List[EntityType], optional
        :param estimated_time_of_arrival: The estimated time period in which the beneficiary will receive the funds., defaults to None
        :type estimated_time_of_arrival: str, optional
        :param status: Indicates whether the payout method is currently available. One of the following values: 0 - Not available. 1 - Available., defaults to None
        :type status: int, optional
        """
        self.amount_range_per_currency = amount_range_per_currency
        self.beneficiary_country = self._define_str(
            "beneficiary_country", beneficiary_country, nullable=True
        )
        self.beneficiary_entity_types = self._define_list(
            beneficiary_entity_types, EntityType
        )
        self.category = (
            self._enum_matching(category, Category.list(), "category")
            if category
            else None
        )
        self.image = self._define_str("image", image, nullable=True)
        self.is_cancelable = self._define_number(
            "is_cancelable", is_cancelable, nullable=True
        )
        self.is_expirable = self._define_number(
            "is_expirable", is_expirable, nullable=True
        )
        self.is_location_specific = self._define_number(
            "is_location_specific", is_location_specific, nullable=True
        )
        self.maximum_expiration_seconds = self._define_number(
            "maximum_expiration_seconds", maximum_expiration_seconds, nullable=True
        )
        self.minimum_expiration_seconds = self._define_number(
            "minimum_expiration_seconds", minimum_expiration_seconds, nullable=True
        )
        self.name = self._define_str("name", name, nullable=True)
        self.payout_currencies = payout_currencies
        self.payout_method_type = self._define_str(
            "payout_method_type", payout_method_type, nullable=True
        )
        self.sender_country = self._define_str(
            "sender_country", sender_country, nullable=True
        )
        self.sender_currencies = sender_currencies
        self.sender_entity_types = self._define_list(sender_entity_types, EntityType)
        self.estimated_time_of_arrival = self._define_str(
            "estimated_time_of_arrival", estimated_time_of_arrival, nullable=True
        )
        self.status = self._define_number("status", status, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class V1paymentssubscriptionsSubscriptionItems(BaseModel):
    """V1paymentssubscriptionsSubscriptionItems

    :param plan: ID of the plan assigned to this subscription item., defaults to None
    :type plan: str, optional
    :param quantity: quantity, defaults to None
    :type quantity: float, optional
    """

    def __init__(self, plan: str = None, quantity: float = None):
        """V1paymentssubscriptionsSubscriptionItems

        :param plan: ID of the plan assigned to this subscription item., defaults to None
        :type plan: str, optional
        :param quantity: quantity, defaults to None
        :type quantity: float, optional
        """
        self.plan = self._define_str("plan", plan, nullable=True)
        self.quantity = self._define_number("quantity", quantity, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"type_": "type"})
class InlineResponse200_112Data(BaseModel):
    """InlineResponse200_112Data

    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document., defaults to None
    :type country: str, optional
    :param type_: Type of the identification document. Two-letter code, defaults to None
    :type type_: str, optional
    :param name: Name of the identification document. Two-letter code, defaults to None
    :type name: str, optional
    :param is_back_required: Boolean if back image required, defaults to None
    :type is_back_required: bool, optional
    :param is_address_extractable: Boolean if address extractable, defaults to None
    :type is_address_extractable: bool, optional
    """

    def __init__(
        self,
        country: str = None,
        type_: str = None,
        name: str = None,
        is_back_required: bool = None,
        is_address_extractable: bool = None,
    ):
        """InlineResponse200_112Data

        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document., defaults to None
        :type country: str, optional
        :param type_: Type of the identification document. Two-letter code, defaults to None
        :type type_: str, optional
        :param name: Name of the identification document. Two-letter code, defaults to None
        :type name: str, optional
        :param is_back_required: Boolean if back image required, defaults to None
        :type is_back_required: bool, optional
        :param is_address_extractable: Boolean if address extractable, defaults to None
        :type is_address_extractable: bool, optional
        """
        self.country = self._define_str("country", country, nullable=True)
        self.type_ = self._define_str("type_", type_, nullable=True)
        self.name = self._define_str("name", name, nullable=True)
        self.is_back_required = is_back_required
        self.is_address_extractable = is_address_extractable



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .field_1 import Field1


@JsonMap({"type_": "type"})
class CustomerRequestPaymentMethod(BaseModel):
    """The payment method that is used when the transaction does not specify a payment method. String starting with **card_** or other_.

    :param fields: Payment Method Type required field, defaults to None
    :type fields: List[Field1], optional
    :param type_: Type of the payment method. For example, it_visa_card, defaults to None
    :type type_: str, optional
    """

    def __init__(self, fields: List[Field1] = None, type_: str = None):
        """The payment method that is used when the transaction does not specify a payment method. String starting with **card_** or other_.

        :param fields: Payment Method Type required field, defaults to None
        :type fields: List[Field1], optional
        :param type_: Type of the payment method. For example, it_visa_card, defaults to None
        :type type_: str, optional
        """
        self.fields = self._define_list(fields, Field1)
        self.type_ = self._define_str("type_", type_, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap(
    {
        "first_part_address": "firstPartAddress",
        "second_part_address": "secondPartAddress",
        "country_code": "countryCode",
    }
)
class AddCardtoGooglePayResponseUserAddress(BaseModel):
    """The base64 encoded string that contains the encrypted object for Push Provisioning. Required for adding the card to the Google Pay wallet by using the Google Pay `pushTokenizeRequest` method. For more information refer to the Google Pay Provisioning developer documentation .Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet.

    :param first_part_address: First part address., defaults to None
    :type first_part_address: str, optional
    :param second_part_address: Second part address., defaults to None
    :type second_part_address: str, optional
    :param country_code: Two character Country code., defaults to None
    :type country_code: str, optional
    :param locality: Locality such as city, town, etc., defaults to None
    :type locality: str, optional
    """

    def __init__(
        self,
        first_part_address: str = None,
        second_part_address: str = None,
        country_code: str = None,
        locality: str = None,
    ):
        """The base64 encoded string that contains the encrypted object for Push Provisioning. Required for adding the card to the Google Pay wallet by using the Google Pay `pushTokenizeRequest` method. For more information refer to the Google Pay Provisioning developer documentation .Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet.

        :param first_part_address: First part address., defaults to None
        :type first_part_address: str, optional
        :param second_part_address: Second part address., defaults to None
        :type second_part_address: str, optional
        :param country_code: Two character Country code., defaults to None
        :type country_code: str, optional
        :param locality: Locality such as city, town, etc., defaults to None
        :type locality: str, optional
        """
        self.first_part_address = self._define_str(
            "first_part_address", first_part_address, nullable=True
        )
        self.second_part_address = self._define_str(
            "second_part_address", second_part_address, nullable=True
        )
        self.country_code = self._define_str(
            "country_code", country_code, nullable=True
        )
        self.locality = self._define_str("locality", locality, nullable=True)



from __future__ import annotations
from enum import Enum
from typing import Union
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .subscription_hosted_page_reponse_custom_elements import (
    SubscriptionHostedPageReponseCustomElements,
)
from .discount import Discount
from .subscription_hosted_page_reponse_merchant_customer_support import (
    SubscriptionHostedPageReponseMerchantCustomerSupport,
)
from .subscription_items import SubscriptionItems


class SubscriptionHostedPageReponseBillingCycleAnchorGuard(OneOfBaseModel):
    class_list = {"str": str, "float": float}


SubscriptionHostedPageReponseBillingCycleAnchor = Union[str, float]


class SubscriptionHostedPageReponseStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACTIVE: "active"
    :vartype ACTIVE: str
    :cvar CANCELED: "canceled"
    :vartype CANCELED: str
    :cvar PASTDUE: "past_due"
    :vartype PASTDUE: str
    :cvar TRIALING: "trialing"
    :vartype TRIALING: str
    :cvar UNPAID: "unpaid"
    :vartype UNPAID: str
    """

    ACTIVE = "active"
    CANCELED = "canceled"
    PASTDUE = "past_due"
    TRIALING = "trialing"
    UNPAID = "unpaid"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                SubscriptionHostedPageReponseStatus._member_map_.values(),
            )
        )


@JsonMap({"id_": "id"})
class SubscriptionHostedPageReponse(BaseModel):
    """SubscriptionHostedPageReponse

    :param billing: Determines the method of billing at the end of the billing cycle. Set to pay_automatically - Rapyd generates a 'payment' object, then attempts to pay it using the designated payment method., defaults to None
    :type billing: str, optional
    :param billing_cycle_anchor: Determines the start of the next full billing cycle, as defined in the plan described in the 'items'. One of the following values:  * **now** - The present day. The next billing cycle starts right now.  * \<em\>Timestamp\</em\> in Unix time - A time in the future, at or after the end of the free trial period, not more than the length of the billing cycle. The current billing cycle will be shorter than all other billing cycles. Relevant to creation of the subscription.  * **unchanged** - The original billing cycle anchor is unchanged. Relevant to updating a subscription. , defaults to None
    :type billing_cycle_anchor: SubscriptionHostedPageReponseBillingCycleAnchor, optional
    :param cancel_at_period_end: Determines the last date that charges accrue. **true** - Charges accrue until the end of the current billing period, then the subscription is canceled.  * When no trial period is set, after `cancel_at_period_end` is set to **true** the subscription will not be renewed at the next interval.  * When a trial period is set after `cancel_at_period_end` is set to **true**, the subscription will not begin. \<BR\> * **false** - This is the default.  * When the subscription is created, no end is defined. * When 'Cancel Subscription' is run, charges stop immediately and the subscription is canceled. , defaults to None
    :type cancel_at_period_end: bool, optional
    :param cancel_checkout_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs. or by the client. Unix time. Response only., defaults to None
    :type cancel_checkout_url: str, optional
    :param canceled_at: Date and time that the subscription is canceled by the customer or by the client. Unix time. Response only., defaults to None
    :type canceled_at: float, optional
    :param complete_checkout_url: URL where the customer is redirected after pressing **Finish** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type complete_checkout_url: str, optional
    :param complete_payment_url: URL where the customer is redirected when payment is successful, after returning from an external page such as a 3DS page. Does not support localhost URLs., defaults to None
    :type complete_payment_url: str, optional
    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
    :type country: str, optional
    :param coupon: ID of a coupon to apply a discount to the subscription. String starting with **coupon_**., defaults to None
    :type coupon: str, optional
    :param created_at: Time of creation of this subscription, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param current_period_end: End of the current billing cycle, in Unix time. Response only., defaults to None
    :type current_period_end: float, optional
    :param current_period_start: Start of the current billing cycle, in Unix time. Response only., defaults to None
    :type current_period_start: float, optional
    :param custom_elements: Description of the payment transaction., defaults to None
    :type custom_elements: SubscriptionHostedPageReponseCustomElements, optional
    :param customer: ID of the customer who pays for this subscription. String starting with cus_. Response only., defaults to None
    :type customer: str, optional
    :param days_until_due: Number of days from the invoice date for customer to complete the payment., defaults to None
    :type days_until_due: float, optional
    :param discount: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
    :type discount: Discount, optional
    :param error_payment_url: URL where the customer is redirected when payment is not successful, after returning from an external page, such as a 3DS page. Does not support localhost URLs., defaults to None
    :type error_payment_url: str, optional
    :param id_: ID of the subscription. String starting with **hp_sub_**., defaults to None
    :type id_: str, optional
    :param language: Determines the default language of the hosted page. For a list of values, see List Supported Languages.\<BR\> * When this parameter is null, the language of the user's browser is used.\<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param merchant_alias: Client's name., defaults to None
    :type merchant_alias: str, optional
    :param merchant_color: Color of the call-to-action (CTA) button on the hosted page.\<BR\> To configure this field, use the Client Portal., defaults to None
    :type merchant_color: str, optional
    :param merchant_customer_support: merchant_customer_support, defaults to None
    :type merchant_customer_support: SubscriptionHostedPageReponseMerchantCustomerSupport, optional
    :param merchant_main_button: Indicates the text for the call-to-action button on the subscription checkout page., defaults to None
    :type merchant_main_button: dict, optional
    :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers.\<BR\> To configure this field, use the Client Portal. See Customizing Your Hosted Page., defaults to None
    :type merchant_privacy_policy: str, optional
    :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Portal., defaults to None
    :type merchant_terms: str, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page.\<BR\> Relevant when one or both of the following fields is unset:\<BR\>* **cancel_url**\<BR\>* **complete_url**\<BR\> To configure this field, use the Client Portal. See Customizing Your Hosted Page.URL for the client's terms and conditions. To configure this field, use the Client Portal., defaults to None
    :type merchant_website: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If page_expiration is not set, the hosted page expires 14 days after creation.\<BR\> * **Range**: 1 minute to 30 days., defaults to None
    :type page_expiration: float, optional
    :param payment: Describes the payment that will result from the hosted page. The id and status values are **null** until the customer successfully submits the information on the hosted page. For more information about the fields in the 'payment' object, see Create Payment., defaults to None
    :type payment: dict, optional
    :param payment_fields: Additional `payment_options` fields., defaults to None
    :type payment_fields: dict, optional
    :param payment_method: Payment method used for paying invoices generated by this subscription. The ID is a string starting with **card_** or **other_**., defaults to None
    :type payment_method: str, optional
    :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
    :type redirect_url: str, optional
    :param status: Status of the subscription. One of the following:  * active - The customer is currently paying for this subscription.  * canceled - The customer has canceled this subscription, but it remains in the Rapyd database.  * past_due - Payment for this subscription was not received by the end of the billing period.  * trialing - The subscription is in its free trial period. * unpaid - An error occurred in the payment for this subscription. Response only. , defaults to None
    :type status: SubscriptionHostedPageReponseStatus, optional
    :param subscription_items: subscription_items, defaults to None
    :type subscription_items: SubscriptionItems, optional
    :param tax_percent: The percentage tax rate that is applied to the subtotal of the invoice, after subtracting all discounts. Decimal. Range: 0-100, with up to four decimal places., defaults to None
    :type tax_percent: float, optional
    :param total_count: Total number of subscription items., defaults to None
    :type total_count: float, optional
    :param trial_end: The date and time of the end of the customer's free trial period, in Unix time, or the string **now**. Takes precedence over `trial_period_days`. If `trial_end` is not set by the client, Rapyd calculates this date by adding `trial_period_days` to the date in `created_at`. Relevant when `trial_period_days` is not defined., defaults to None
    :type trial_end: float, optional
    :param trial_from_plan: Determines whether a free trial period can be defined in a plan attached to the subscription., defaults to None
    :type trial_from_plan: bool, optional
    :param trial_period_days: The number of days in the customer's free trial period. Integer. Range: 0-730. This value takes precedence over trial periods that are defined in a plan. Relevant when `trial_end` is not defined., defaults to None
    :type trial_period_days: float, optional
    :param trial_start: Date and time of the start of the customer's free trial period, in Unix time., defaults to None
    :type trial_start: float, optional
    """

    def __init__(
        self,
        billing: str = None,
        billing_cycle_anchor: SubscriptionHostedPageReponseBillingCycleAnchor = None,
        cancel_at_period_end: bool = None,
        cancel_checkout_url: str = None,
        canceled_at: float = None,
        complete_checkout_url: str = None,
        complete_payment_url: str = None,
        country: str = None,
        coupon: str = None,
        created_at: float = None,
        current_period_end: float = None,
        current_period_start: float = None,
        custom_elements: SubscriptionHostedPageReponseCustomElements = None,
        customer: str = None,
        days_until_due: float = None,
        discount: Discount = None,
        error_payment_url: str = None,
        id_: str = None,
        language: str = None,
        merchant_alias: str = None,
        merchant_color: str = None,
        merchant_customer_support: SubscriptionHostedPageReponseMerchantCustomerSupport = None,
        merchant_main_button: dict = None,
        merchant_privacy_policy: str = None,
        merchant_reference_id: str = None,
        merchant_terms: str = None,
        merchant_website: str = None,
        metadata: dict = None,
        page_expiration: float = None,
        payment: dict = None,
        payment_fields: dict = None,
        payment_method: str = None,
        redirect_url: str = None,
        status: SubscriptionHostedPageReponseStatus = None,
        subscription_items: SubscriptionItems = None,
        tax_percent: float = None,
        total_count: float = None,
        trial_end: float = None,
        trial_from_plan: bool = None,
        trial_period_days: float = None,
        trial_start: float = None,
    ):
        """SubscriptionHostedPageReponse

        :param billing: Determines the method of billing at the end of the billing cycle. Set to pay_automatically - Rapyd generates a 'payment' object, then attempts to pay it using the designated payment method., defaults to None
        :type billing: str, optional
        :param billing_cycle_anchor: Determines the start of the next full billing cycle, as defined in the plan described in the 'items'. One of the following values:  * **now** - The present day. The next billing cycle starts right now.  * \<em\>Timestamp\</em\> in Unix time - A time in the future, at or after the end of the free trial period, not more than the length of the billing cycle. The current billing cycle will be shorter than all other billing cycles. Relevant to creation of the subscription.  * **unchanged** - The original billing cycle anchor is unchanged. Relevant to updating a subscription. , defaults to None
        :type billing_cycle_anchor: SubscriptionHostedPageReponseBillingCycleAnchor, optional
        :param cancel_at_period_end: Determines the last date that charges accrue. **true** - Charges accrue until the end of the current billing period, then the subscription is canceled.  * When no trial period is set, after `cancel_at_period_end` is set to **true** the subscription will not be renewed at the next interval.  * When a trial period is set after `cancel_at_period_end` is set to **true**, the subscription will not begin. \<BR\> * **false** - This is the default.  * When the subscription is created, no end is defined. * When 'Cancel Subscription' is run, charges stop immediately and the subscription is canceled. , defaults to None
        :type cancel_at_period_end: bool, optional
        :param cancel_checkout_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs. or by the client. Unix time. Response only., defaults to None
        :type cancel_checkout_url: str, optional
        :param canceled_at: Date and time that the subscription is canceled by the customer or by the client. Unix time. Response only., defaults to None
        :type canceled_at: float, optional
        :param complete_checkout_url: URL where the customer is redirected after pressing **Finish** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type complete_checkout_url: str, optional
        :param complete_payment_url: URL where the customer is redirected when payment is successful, after returning from an external page such as a 3DS page. Does not support localhost URLs., defaults to None
        :type complete_payment_url: str, optional
        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
        :type country: str, optional
        :param coupon: ID of a coupon to apply a discount to the subscription. String starting with **coupon_**., defaults to None
        :type coupon: str, optional
        :param created_at: Time of creation of this subscription, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param current_period_end: End of the current billing cycle, in Unix time. Response only., defaults to None
        :type current_period_end: float, optional
        :param current_period_start: Start of the current billing cycle, in Unix time. Response only., defaults to None
        :type current_period_start: float, optional
        :param custom_elements: Description of the payment transaction., defaults to None
        :type custom_elements: SubscriptionHostedPageReponseCustomElements, optional
        :param customer: ID of the customer who pays for this subscription. String starting with cus_. Response only., defaults to None
        :type customer: str, optional
        :param days_until_due: Number of days from the invoice date for customer to complete the payment., defaults to None
        :type days_until_due: float, optional
        :param discount: Describes the fields relating to discounts in REST messages and webhooks for customer profiles and subscriptions Contains information about the coupon that applies to the customer. Read-only field. Adding a discount is a 2-step process: \<BR\> 1. Create Coupon, which returns a coupon ID. \<BR\>2. Add the coupon ID to the coupon field of the customer with Create Customer or Update Customer., defaults to None
        :type discount: Discount, optional
        :param error_payment_url: URL where the customer is redirected when payment is not successful, after returning from an external page, such as a 3DS page. Does not support localhost URLs., defaults to None
        :type error_payment_url: str, optional
        :param id_: ID of the subscription. String starting with **hp_sub_**., defaults to None
        :type id_: str, optional
        :param language: Determines the default language of the hosted page. For a list of values, see List Supported Languages.\<BR\> * When this parameter is null, the language of the user's browser is used.\<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param merchant_alias: Client's name., defaults to None
        :type merchant_alias: str, optional
        :param merchant_color: Color of the call-to-action (CTA) button on the hosted page.\<BR\> To configure this field, use the Client Portal., defaults to None
        :type merchant_color: str, optional
        :param merchant_customer_support: merchant_customer_support, defaults to None
        :type merchant_customer_support: SubscriptionHostedPageReponseMerchantCustomerSupport, optional
        :param merchant_main_button: Indicates the text for the call-to-action button on the subscription checkout page., defaults to None
        :type merchant_main_button: dict, optional
        :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers.\<BR\> To configure this field, use the Client Portal. See Customizing Your Hosted Page., defaults to None
        :type merchant_privacy_policy: str, optional
        :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Portal., defaults to None
        :type merchant_terms: str, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page.\<BR\> Relevant when one or both of the following fields is unset:\<BR\>* **cancel_url**\<BR\>* **complete_url**\<BR\> To configure this field, use the Client Portal. See Customizing Your Hosted Page.URL for the client's terms and conditions. To configure this field, use the Client Portal., defaults to None
        :type merchant_website: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If page_expiration is not set, the hosted page expires 14 days after creation.\<BR\> * **Range**: 1 minute to 30 days., defaults to None
        :type page_expiration: float, optional
        :param payment: Describes the payment that will result from the hosted page. The id and status values are **null** until the customer successfully submits the information on the hosted page. For more information about the fields in the 'payment' object, see Create Payment., defaults to None
        :type payment: dict, optional
        :param payment_fields: Additional `payment_options` fields., defaults to None
        :type payment_fields: dict, optional
        :param payment_method: Payment method used for paying invoices generated by this subscription. The ID is a string starting with **card_** or **other_**., defaults to None
        :type payment_method: str, optional
        :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
        :type redirect_url: str, optional
        :param status: Status of the subscription. One of the following:  * active - The customer is currently paying for this subscription.  * canceled - The customer has canceled this subscription, but it remains in the Rapyd database.  * past_due - Payment for this subscription was not received by the end of the billing period.  * trialing - The subscription is in its free trial period. * unpaid - An error occurred in the payment for this subscription. Response only. , defaults to None
        :type status: SubscriptionHostedPageReponseStatus, optional
        :param subscription_items: subscription_items, defaults to None
        :type subscription_items: SubscriptionItems, optional
        :param tax_percent: The percentage tax rate that is applied to the subtotal of the invoice, after subtracting all discounts. Decimal. Range: 0-100, with up to four decimal places., defaults to None
        :type tax_percent: float, optional
        :param total_count: Total number of subscription items., defaults to None
        :type total_count: float, optional
        :param trial_end: The date and time of the end of the customer's free trial period, in Unix time, or the string **now**. Takes precedence over `trial_period_days`. If `trial_end` is not set by the client, Rapyd calculates this date by adding `trial_period_days` to the date in `created_at`. Relevant when `trial_period_days` is not defined., defaults to None
        :type trial_end: float, optional
        :param trial_from_plan: Determines whether a free trial period can be defined in a plan attached to the subscription., defaults to None
        :type trial_from_plan: bool, optional
        :param trial_period_days: The number of days in the customer's free trial period. Integer. Range: 0-730. This value takes precedence over trial periods that are defined in a plan. Relevant when `trial_end` is not defined., defaults to None
        :type trial_period_days: float, optional
        :param trial_start: Date and time of the start of the customer's free trial period, in Unix time., defaults to None
        :type trial_start: float, optional
        """
        self.billing = self._define_str("billing", billing, nullable=True)
        self.billing_cycle_anchor = (
            SubscriptionHostedPageReponseBillingCycleAnchorGuard.return_one_of(
                billing_cycle_anchor
            )
        )
        self.cancel_at_period_end = cancel_at_period_end
        self.cancel_checkout_url = self._define_str(
            "cancel_checkout_url", cancel_checkout_url, nullable=True
        )
        self.canceled_at = self._define_number(
            "canceled_at", canceled_at, nullable=True
        )
        self.complete_checkout_url = self._define_str(
            "complete_checkout_url", complete_checkout_url, nullable=True
        )
        self.complete_payment_url = self._define_str(
            "complete_payment_url", complete_payment_url, nullable=True
        )
        self.country = self._define_str("country", country, nullable=True)
        self.coupon = self._define_str("coupon", coupon, nullable=True)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.current_period_end = self._define_number(
            "current_period_end", current_period_end, nullable=True
        )
        self.current_period_start = self._define_number(
            "current_period_start", current_period_start, nullable=True
        )
        self.custom_elements = self._define_object(
            custom_elements, SubscriptionHostedPageReponseCustomElements
        )
        self.customer = self._define_str("customer", customer, nullable=True)
        self.days_until_due = self._define_number(
            "days_until_due", days_until_due, nullable=True
        )
        self.discount = self._define_object(discount, Discount)
        self.error_payment_url = self._define_str(
            "error_payment_url", error_payment_url, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.language = self._define_str("language", language, nullable=True)
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_customer_support = self._define_object(
            merchant_customer_support,
            SubscriptionHostedPageReponseMerchantCustomerSupport,
        )
        self.merchant_main_button = merchant_main_button
        self.merchant_privacy_policy = self._define_str(
            "merchant_privacy_policy", merchant_privacy_policy, nullable=True
        )
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.merchant_terms = self._define_str(
            "merchant_terms", merchant_terms, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.metadata = metadata
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.payment = payment
        self.payment_fields = payment_fields
        self.payment_method = self._define_str(
            "payment_method", payment_method, nullable=True
        )
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.status = (
            self._enum_matching(
                status, SubscriptionHostedPageReponseStatus.list(), "status"
            )
            if status
            else None
        )
        self.subscription_items = self._define_object(
            subscription_items, SubscriptionItems
        )
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )
        self.total_count = self._define_number(
            "total_count", total_count, nullable=True
        )
        self.trial_end = self._define_number("trial_end", trial_end, nullable=True)
        self.trial_from_plan = trial_from_plan
        self.trial_period_days = self._define_number(
            "trial_period_days", trial_period_days, nullable=True
        )
        self.trial_start = self._define_number(
            "trial_start", trial_start, nullable=True
        )



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .product_package_dimensions import ProductPackageDimensions
from .sku import Sku


class ProductType(Enum):
    """An enumeration representing different categories.

    :cvar GOODS: "goods"
    :vartype GOODS: str
    :cvar SERVICES: "services"
    :vartype SERVICES: str
    """

    GOODS = "goods"
    SERVICES = "services"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ProductType._member_map_.values()))


@JsonMap({"id_": "id", "type_": "type"})
class Product(BaseModel):
    """Product

    :param active: Indicates whether the product is currently available for purchase., defaults to None
    :type active: bool, optional
    :param attributes: Up to 5 unique alphanumeric strings defined by the merchant. Cannot contain spaces. For example: [ \> "size", \> "color"] Each string defines the key in a key-value pair in the 'attributes' object in the corresponding 'sku' objects., defaults to None
    :type attributes: List[str], optional
    :param created_at: Time of creation of this product, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param description: Full text description of the product., defaults to None
    :type description: str, optional
    :param id_: Unique string for identification of the product. Legal input includes alphanumeric characters and the underscore () character. If the merchant does not define an ID, Rapyd generates it with a string that starts with **product**., defaults to None
    :type id_: str, optional
    :param images: Images associated with the product., defaults to None
    :type images: List[str], optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param name: The name of the product or service that appears in the line items in invoices., defaults to None
    :type name: str, optional
    :param package_dimensions: Describes the physical size and weight of the product. Contains the following fields:  * height  * length  * weight  * width  These fields are represented as numbers, but it is the responsibility of the merchant to define and interpret the relevant units of length and weight. , defaults to None
    :type package_dimensions: ProductPackageDimensions, optional
    :param shippable: Indicates whether the product can be shipped., defaults to None
    :type shippable: bool, optional
    :param skus: skus, defaults to None
    :type skus: List[Sku], optional
    :param statement_descriptor: Description that is suitable for a customer's statement. Limited to 22 characters. Relevant when type is service. Must be null when type is goods., defaults to None
    :type statement_descriptor: str, optional
    :param type_: One of the following:  * services - Relevant to subscriptions and plans.  * goods - Relevant to orders and SKUs. , defaults to None
    :type type_: ProductType, optional
    :param unit_label: Determines what one unit of this product is called on customersâ€™ receipts and invoices, such as minutes, viewings, kilometers or packages. Relevant when type is service. Must be null when type is goods., defaults to None
    :type unit_label: str, optional
    :param updated_at: Time that this product was last updated, in Unix time. Response only., defaults to None
    :type updated_at: float, optional
    """

    def __init__(
        self,
        active: bool = None,
        attributes: List[str] = None,
        created_at: float = None,
        description: str = None,
        id_: str = None,
        images: List[str] = None,
        metadata: dict = None,
        name: str = None,
        package_dimensions: ProductPackageDimensions = None,
        shippable: bool = None,
        skus: List[Sku] = None,
        statement_descriptor: str = None,
        type_: ProductType = None,
        unit_label: str = None,
        updated_at: float = None,
    ):
        """Product

        :param active: Indicates whether the product is currently available for purchase., defaults to None
        :type active: bool, optional
        :param attributes: Up to 5 unique alphanumeric strings defined by the merchant. Cannot contain spaces. For example: [ \> "size", \> "color"] Each string defines the key in a key-value pair in the 'attributes' object in the corresponding 'sku' objects., defaults to None
        :type attributes: List[str], optional
        :param created_at: Time of creation of this product, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param description: Full text description of the product., defaults to None
        :type description: str, optional
        :param id_: Unique string for identification of the product. Legal input includes alphanumeric characters and the underscore () character. If the merchant does not define an ID, Rapyd generates it with a string that starts with **product**., defaults to None
        :type id_: str, optional
        :param images: Images associated with the product., defaults to None
        :type images: List[str], optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param name: The name of the product or service that appears in the line items in invoices., defaults to None
        :type name: str, optional
        :param package_dimensions: Describes the physical size and weight of the product. Contains the following fields:  * height  * length  * weight  * width  These fields are represented as numbers, but it is the responsibility of the merchant to define and interpret the relevant units of length and weight. , defaults to None
        :type package_dimensions: ProductPackageDimensions, optional
        :param shippable: Indicates whether the product can be shipped., defaults to None
        :type shippable: bool, optional
        :param skus: skus, defaults to None
        :type skus: List[Sku], optional
        :param statement_descriptor: Description that is suitable for a customer's statement. Limited to 22 characters. Relevant when type is service. Must be null when type is goods., defaults to None
        :type statement_descriptor: str, optional
        :param type_: One of the following:  * services - Relevant to subscriptions and plans.  * goods - Relevant to orders and SKUs. , defaults to None
        :type type_: ProductType, optional
        :param unit_label: Determines what one unit of this product is called on customersâ€™ receipts and invoices, such as minutes, viewings, kilometers or packages. Relevant when type is service. Must be null when type is goods., defaults to None
        :type unit_label: str, optional
        :param updated_at: Time that this product was last updated, in Unix time. Response only., defaults to None
        :type updated_at: float, optional
        """
        self.active = active
        self.attributes = attributes
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.description = self._define_str("description", description, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.images = images
        self.metadata = metadata
        self.name = self._define_str("name", name, nullable=True)
        self.package_dimensions = self._define_object(
            package_dimensions, ProductPackageDimensions
        )
        self.shippable = shippable
        self.skus = self._define_list(skus, Sku)
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )
        self.type_ = (
            self._enum_matching(type_, ProductType.list(), "type_") if type_ else None
        )
        self.unit_label = self._define_str("unit_label", unit_label, nullable=True)
        self.updated_at = self._define_number("updated_at", updated_at, nullable=True)



from enum import Enum


class Category(Enum):
    """An enumeration representing different categories.

    :cvar BANK: "bank"
    :vartype BANK: str
    :cvar CARD: "card"
    :vartype CARD: str
    :cvar CASH: "cash"
    :vartype CASH: str
    :cvar EWALLET: "ewallet"
    :vartype EWALLET: str
    :cvar RAPYDEWALLET: "rapyd_ewallet"
    :vartype RAPYDEWALLET: str
    :cvar BANKREDIRECT: "bank_redirect"
    :vartype BANKREDIRECT: str
    :cvar BANKTRANSFER: "bank_transfer"
    :vartype BANKTRANSFER: str
    """

    BANK = "bank"
    CARD = "card"
    CASH = "cash"
    EWALLET = "ewallet"
    RAPYDEWALLET = "rapyd_ewallet"
    BANKREDIRECT = "bank_redirect"
    BANKTRANSFER = "bank_transfer"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Category._member_map_.values()))



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class TransferStatus(Enum):
    """An enumeration representing different categories.

    :cvar CAN: "CAN"
    :vartype CAN: str
    :cvar CLO: "CLO"
    :vartype CLO: str
    :cvar DEC: "DEC"
    :vartype DEC: str
    :cvar EXP: "EXP"
    :vartype EXP: str
    :cvar HLD: "HLD"
    :vartype HLD: str
    :cvar PEN: "PEN"
    :vartype PEN: str
    :cvar REJ: "REJ"
    :vartype REJ: str
    """

    CAN = "CAN"
    CLO = "CLO"
    DEC = "DEC"
    EXP = "EXP"
    HLD = "HLD"
    PEN = "PEN"
    REJ = "REJ"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, TransferStatus._member_map_.values()))


@JsonMap({"id_": "id"})
class Transfer(BaseModel):
    """Transfer

    :param amount: Amount of the transfer. Decimal., defaults to None
    :type amount: float, optional
    :param created_at: Time the transaction was made, in Unix time., defaults to None
    :type created_at: float, optional
    :param currency_code: Three-letter ISO 4217 code for the currency used in the amount field., defaults to None
    :type currency_code: str, optional
    :param destination_ewallet_id: ID of the wallet receiving the money. String starting with **ewallet_**. Response only., defaults to None
    :type destination_ewallet_id: str, optional
    :param destination_phone_number: Phone number of the owner of the wallet receiving the money, in E.164 format., defaults to None
    :type destination_phone_number: str, optional
    :param destination_transaction_id: ID of the transaction with regard to the destination. String starting with **wt_**., defaults to None
    :type destination_transaction_id: str, optional
    :param id_: ID of the transaction. String starting with **wt_** or UUID., defaults to None
    :type id_: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param response_metadata: Metadata created with Set Transfer Response. Response only., defaults to None
    :type response_metadata: dict, optional
    :param source_ewallet_id: ID of the wallet sending the money. String starting with **ewallet_**. Response only., defaults to None
    :type source_ewallet_id: str, optional
    :param source_transaction_id: ID of the transaction with regard to the source. String starting with **wt_**., defaults to None
    :type source_transaction_id: str, optional
    :param status: \< Status of the transaction. CAN - Canceled. The transferor canceled the transfer. CLO - Closed. The transferee accepted the funds. DEC - Declined. The transferee rejected the transfer. EXP - Expired. The transferee did not respond before the transfer expired. HLD - Hold. Rapyd Protect is putting this transfer on hold and reviewing it. PEN - Pending. Waiting for the transferee to accept. REJ - Rejected. Rapyd Protect has rejected this transfer., defaults to None
    :type status: TransferStatus, optional
    :param transfer_response_at: Time of the Set Transfer Response operation, in Unix time. Read-only., defaults to None
    :type transfer_response_at: float, optional
    :param expiration: Determines the day the transfer expires, in Unix time. Acceptance of the transfer must occur before the start of this day. Default is 14 days after creation of the transfer., defaults to None
    :type expiration: float, optional
    """

    def __init__(
        self,
        amount: float = None,
        created_at: float = None,
        currency_code: str = None,
        destination_ewallet_id: str = None,
        destination_phone_number: str = None,
        destination_transaction_id: str = None,
        id_: str = None,
        metadata: dict = None,
        response_metadata: dict = None,
        source_ewallet_id: str = None,
        source_transaction_id: str = None,
        status: TransferStatus = None,
        transfer_response_at: float = None,
        expiration: float = None,
    ):
        """Transfer

        :param amount: Amount of the transfer. Decimal., defaults to None
        :type amount: float, optional
        :param created_at: Time the transaction was made, in Unix time., defaults to None
        :type created_at: float, optional
        :param currency_code: Three-letter ISO 4217 code for the currency used in the amount field., defaults to None
        :type currency_code: str, optional
        :param destination_ewallet_id: ID of the wallet receiving the money. String starting with **ewallet_**. Response only., defaults to None
        :type destination_ewallet_id: str, optional
        :param destination_phone_number: Phone number of the owner of the wallet receiving the money, in E.164 format., defaults to None
        :type destination_phone_number: str, optional
        :param destination_transaction_id: ID of the transaction with regard to the destination. String starting with **wt_**., defaults to None
        :type destination_transaction_id: str, optional
        :param id_: ID of the transaction. String starting with **wt_** or UUID., defaults to None
        :type id_: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param response_metadata: Metadata created with Set Transfer Response. Response only., defaults to None
        :type response_metadata: dict, optional
        :param source_ewallet_id: ID of the wallet sending the money. String starting with **ewallet_**. Response only., defaults to None
        :type source_ewallet_id: str, optional
        :param source_transaction_id: ID of the transaction with regard to the source. String starting with **wt_**., defaults to None
        :type source_transaction_id: str, optional
        :param status: \< Status of the transaction. CAN - Canceled. The transferor canceled the transfer. CLO - Closed. The transferee accepted the funds. DEC - Declined. The transferee rejected the transfer. EXP - Expired. The transferee did not respond before the transfer expired. HLD - Hold. Rapyd Protect is putting this transfer on hold and reviewing it. PEN - Pending. Waiting for the transferee to accept. REJ - Rejected. Rapyd Protect has rejected this transfer., defaults to None
        :type status: TransferStatus, optional
        :param transfer_response_at: Time of the Set Transfer Response operation, in Unix time. Read-only., defaults to None
        :type transfer_response_at: float, optional
        :param expiration: Determines the day the transfer expires, in Unix time. Acceptance of the transfer must occur before the start of this day. Default is 14 days after creation of the transfer., defaults to None
        :type expiration: float, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency_code = self._define_str(
            "currency_code", currency_code, nullable=True
        )
        self.destination_ewallet_id = self._define_str(
            "destination_ewallet_id", destination_ewallet_id, nullable=True
        )
        self.destination_phone_number = self._define_str(
            "destination_phone_number", destination_phone_number, nullable=True
        )
        self.destination_transaction_id = self._define_str(
            "destination_transaction_id", destination_transaction_id, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.metadata = metadata
        self.response_metadata = response_metadata
        self.source_ewallet_id = self._define_str(
            "source_ewallet_id", source_ewallet_id, nullable=True
        )
        self.source_transaction_id = self._define_str(
            "source_transaction_id", source_transaction_id, nullable=True
        )
        self.status = (
            self._enum_matching(status, TransferStatus.list(), "status")
            if status
            else None
        )
        self.transfer_response_at = self._define_number(
            "transfer_response_at", transfer_response_at, nullable=True
        )
        self.expiration = self._define_number("expiration", expiration, nullable=True)



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import InlineResponse200_105, InlineResponse200_106


class WebhookMethodsService(BaseService):

    @cast_models
    def get_webhook(
        self,
        webhook: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_105:
        """Retrieve a webhook. Use 'List Webhooks' to find the IDs of webhooks.

        :param webhook: The webhook ID. String starting with **wh_**.
        :type webhook: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieves webhook
        :rtype: InlineResponse200_105
        """

        Validator(str).validate(webhook)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/webhooks/{{webhook}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("webhook", webhook, explode=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_105._unmap(response)

    @cast_models
    def resend_webhook(
        self,
        webhook: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_105:
        """Resend a webhook that was not sent successfully. Use 'List Webhooks' to find the IDs of webhooks. You can resend a webhook that is in status **ERR**.

        :param webhook: The webhook ID. String starting with **wh_**.
        :type webhook: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Resends webhook
        :rtype: InlineResponse200_105
        """

        Validator(str).validate(webhook)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/webhooks/{{webhook}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("webhook", webhook, explode=True)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_105._unmap(response)

    @cast_models
    def list_webhooks(
        self,
        from_created_at: str = None,
        limit: float = None,
        page: str = None,
        status: str = None,
        type_: str = None,
        to_created_at: float = None,
    ) -> InlineResponse200_106:
        """Retrieve a list of all webhooks that were sent. You can filter the list with query parameters.

        :param from_created_at: The earliest date and time when the object was created, in Unix time (seconds)., defaults to None
        :type from_created_at: str, optional
        :param limit: The maximum number of objects to return. Range: 1-1000., defaults to None
        :type limit: float, optional
        :param page: Page number for pagination., defaults to None
        :type page: str, optional
        :param status: Status of the webhook. One of the following: <BR> * **NEW** (new) - The webhook was created and has not yet been sent successfully. <BR> * **CLO** (closed) - The webhook was sent successfully.<BR> * **ERR** (error) - Attempts were made to send the webhook, but the maximum number of retries was  reached. The automatic retry process failed. The webhook was not sent. <BR> * **RET** (retried) - The webhook was resent., defaults to None
        :type status: str, optional
        :param type_: The type of webhook., defaults to None
        :type type_: str, optional
        :param to_created_at: The latest date and time when the object was created, in Unix time (seconds)., defaults to None
        :type to_created_at: float, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Resends webhook
        :rtype: InlineResponse200_106
        """

        Validator(str).is_optional().validate(from_created_at)
        Validator(float).is_optional().validate(limit)
        Validator(str).is_optional().validate(page)
        Validator(str).is_optional().validate(status)
        Validator(str).is_optional().validate(type_)
        Validator(float).is_optional().validate(to_created_at)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/webhooks", self.get_default_headers())
            .add_query("from_created_at", from_created_at)
            .add_query("limit", limit)
            .add_query("page", page)
            .add_query("status", status)
            .add_query("type", type_)
            .add_query("to_created_at", to_created_at)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_106._unmap(response)



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class PutFundsOnHoldResponseDestinationBalanceType(Enum):
    """An enumeration representing different categories.

    :cvar AVAILABLEBALANCE: "available_balance"
    :vartype AVAILABLEBALANCE: str
    :cvar ONHOLDBALANCE: "on_hold_balance"
    :vartype ONHOLDBALANCE: str
    :cvar RECEIVEDBALANCE: "received_balance"
    :vartype RECEIVEDBALANCE: str
    :cvar RESERVEBALANCE: "reserve_balance"
    :vartype RESERVEBALANCE: str
    """

    AVAILABLEBALANCE = "available_balance"
    ONHOLDBALANCE = "on_hold_balance"
    RECEIVEDBALANCE = "received_balance"
    RESERVEBALANCE = "reserve_balance"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                PutFundsOnHoldResponseDestinationBalanceType._member_map_.values(),
            )
        )


class PutFundsOnHoldResponseSourceBalanceType(Enum):
    """An enumeration representing different categories.

    :cvar AVAILABLEBALANCE: "available_balance"
    :vartype AVAILABLEBALANCE: str
    :cvar ONHOLDBALANCE: "on_hold_balance"
    :vartype ONHOLDBALANCE: str
    :cvar RECEIVEDBALANCE: "received_balance"
    :vartype RECEIVEDBALANCE: str
    :cvar RESERVEBALANCE: "reserve_balance"
    :vartype RESERVEBALANCE: str
    """

    AVAILABLEBALANCE = "available_balance"
    ONHOLDBALANCE = "on_hold_balance"
    RECEIVEDBALANCE = "received_balance"
    RESERVEBALANCE = "reserve_balance"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                PutFundsOnHoldResponseSourceBalanceType._member_map_.values(),
            )
        )


@JsonMap({"id_": "id"})
class PutFundsOnHoldResponse(BaseModel):
    """PutFundsOnHoldResponse

    :param amount:  * **Transactions** - Amount of the transaction, in units of the currency defined in `currency`. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 4217:2015.\<BR\> * **Wallet Account Operations** - Amount of the account limit., defaults to None
    :type amount: float, optional
    :param currency_code: Three-letter ISO 4217 code for the currency used in the `amount` field., defaults to None
    :type currency_code: str, optional
    :param destination_account_id: ID of the wallet owner's user. UUID., defaults to None
    :type destination_account_id: str, optional
    :param destination_balance_type: Balance type that the funds are transferred to. See 'Wallet Balance Types'., defaults to None
    :type destination_balance_type: PutFundsOnHoldResponseDestinationBalanceType, optional
    :param destination_transaction_id: ID of the transaction with regard to the destination. String starting with **wt_**., defaults to None
    :type destination_transaction_id: str, optional
    :param destination_user_profile_id: ID of the wallet owner's user. UUID., defaults to None
    :type destination_user_profile_id: str, optional
    :param id_: ID of the transaction. UUID., defaults to None
    :type id_: str, optional
    :param source_balance_type: Balance type that the funds are transferred from. See 'Wallet Balance Types'. See 'Wallet Balance Types'., defaults to None
    :type source_balance_type: PutFundsOnHoldResponseSourceBalanceType, optional
    :param source_transaction_id: ID of the transaction with regard to the source. String starting with **wt_**., defaults to None
    :type source_transaction_id: dict, optional
    :param source_user_profile_id: ID of the wallet owner's user. UUID., defaults to None
    :type source_user_profile_id: str, optional
    """

    def __init__(
        self,
        amount: float = None,
        currency_code: str = None,
        destination_account_id: str = None,
        destination_balance_type: PutFundsOnHoldResponseDestinationBalanceType = None,
        destination_transaction_id: str = None,
        destination_user_profile_id: str = None,
        id_: str = None,
        source_balance_type: PutFundsOnHoldResponseSourceBalanceType = None,
        source_transaction_id: dict = None,
        source_user_profile_id: str = None,
    ):
        """PutFundsOnHoldResponse

        :param amount:  * **Transactions** - Amount of the transaction, in units of the currency defined in `currency`. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 4217:2015.\<BR\> * **Wallet Account Operations** - Amount of the account limit., defaults to None
        :type amount: float, optional
        :param currency_code: Three-letter ISO 4217 code for the currency used in the `amount` field., defaults to None
        :type currency_code: str, optional
        :param destination_account_id: ID of the wallet owner's user. UUID., defaults to None
        :type destination_account_id: str, optional
        :param destination_balance_type: Balance type that the funds are transferred to. See 'Wallet Balance Types'., defaults to None
        :type destination_balance_type: PutFundsOnHoldResponseDestinationBalanceType, optional
        :param destination_transaction_id: ID of the transaction with regard to the destination. String starting with **wt_**., defaults to None
        :type destination_transaction_id: str, optional
        :param destination_user_profile_id: ID of the wallet owner's user. UUID., defaults to None
        :type destination_user_profile_id: str, optional
        :param id_: ID of the transaction. UUID., defaults to None
        :type id_: str, optional
        :param source_balance_type: Balance type that the funds are transferred from. See 'Wallet Balance Types'. See 'Wallet Balance Types'., defaults to None
        :type source_balance_type: PutFundsOnHoldResponseSourceBalanceType, optional
        :param source_transaction_id: ID of the transaction with regard to the source. String starting with **wt_**., defaults to None
        :type source_transaction_id: dict, optional
        :param source_user_profile_id: ID of the wallet owner's user. UUID., defaults to None
        :type source_user_profile_id: str, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.currency_code = self._define_str(
            "currency_code", currency_code, nullable=True
        )
        self.destination_account_id = self._define_str(
            "destination_account_id", destination_account_id, nullable=True
        )
        self.destination_balance_type = (
            self._enum_matching(
                destination_balance_type,
                PutFundsOnHoldResponseDestinationBalanceType.list(),
                "destination_balance_type",
            )
            if destination_balance_type
            else None
        )
        self.destination_transaction_id = self._define_str(
            "destination_transaction_id", destination_transaction_id, nullable=True
        )
        self.destination_user_profile_id = self._define_str(
            "destination_user_profile_id", destination_user_profile_id, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.source_balance_type = (
            self._enum_matching(
                source_balance_type,
                PutFundsOnHoldResponseSourceBalanceType.list(),
                "source_balance_type",
            )
            if source_balance_type
            else None
        )
        self.source_transaction_id = source_transaction_id
        self.source_user_profile_id = self._define_str(
            "source_user_profile_id", source_user_profile_id, nullable=True
        )



from typing import Any, List, Dict, Optional, Union, Type, TypeVar, get_origin, get_args

T = TypeVar("T")


class OneOfBaseModel:
    """
    A base class for handling 'oneOf' models where multiple class constructors are available,
    and the appropriate one is determined based on the input data.

    :ivar dict class_list: A dictionary mapping class names to their constructors.
    """

    class_list = {}

    @classmethod
    def return_one_of(cls, input_data: Optional[Any]) -> Optional[Any]:
        """
        Attempts to initialize an instance of one of the classes in the class_list
        based on the provided input data.

        :param input_data: Input data used for initialization.
        :return: An instance of one of the classes specified.
        :rtype: object
        :raises ValueError: If no class can be initialized with the provided input data,
            or if optional parameters don't match the input data.
        """
        if input_data is None:
            return None

        if isinstance(input_data, (str, float, int, bool)):
            return input_data

        for class_constructor in cls.class_list.values():
            exception_list = []
            try:
                # Check if the class is a only a TypeHint
                origin = get_origin(class_constructor)
                if origin is not None and isinstance(input_data, list):
                    list_instance = cls._get_list_instance(
                        input_data, class_constructor, origin
                    )
                    if list_instance is not None:
                        return list_instance
                    else:
                        continue  # Try the next class constructor

                # Check if the input_data is already an instance of the class
                if isinstance(input_data, class_constructor):
                    return input_data

                # Check if the input_data is a dictionary that can be used to initialize the class
                elif isinstance(input_data, dict):
                    instance = class_constructor._unmap(input_data)
                    if cls._check_params(input_data, instance):
                        return instance
            except Exception as e:
                exception_list.append({"class": class_constructor, "exception": e})

        cls._raise_one_of_error(exception_list)

    @classmethod
    def _get_list_instance(
        cls, input_data: List[Any], class_constructor: Type[T], origin: Type[Any]
    ) -> Union[List[T], List[Any], None]:
        """
        Return the list of elements for a given class constructor and origin type.

        :param input_data: The input data to check.
        :param class_constructor: The constructor of the class to check against.
        :param origin: The origin type to check against.
        :return: The input data if all elements are instances of the type specified in the class_constructor,
            or a new list with each item unmapped.
        """
        args = get_args(class_constructor)
        has_single_arg = args and len(args) == 1

        if not isinstance(input_data, origin) or not has_single_arg:
            return None

        inner_type = args[0]

        if not all(isinstance(item, inner_type) for item in input_data):
            return [inner_type._unmap(item) for item in input_data]

        return input_data

    @classmethod
    def _check_params(cls, raw_input: Dict[str, Any], instance: Any) -> bool:
        """
        Checks if the optional parameters in the instance match the keys in the input data,
        ensuring compliance with one of the models.

        :param dict raw_input: Input data used for initialization.
        :param object instance: An instance of one of the classes specified in the 'oneOf' model.
        :return: True if the optional parameters in the instance match the keys in the input data, False otherwise.
        """
        input_values = {k: v for k, v in raw_input.items() if v is not None}.values()
        instance_map = instance._map()
        instance_values = {
            k: v for k, v in instance_map.items() if v is not None
        }.values()
        return len(input_values) == len(instance_values)

    @classmethod
    def _raise_one_of_error(cls, exception_list):
        """
        Raises a ValueError with the appropriate error message for one of models.

        :param exception_list: List of exceptions that occurred.
        :type exception_list: list
        :raises ValueError: If input data does not match any of the models.
        """
        if not exception_list:
            return
        exception_messages = "\n".join(
            f"Class: {exception['class']}, Exception: {exception['exception']}"
            for exception in exception_list
        )
        raise ValueError(
            f"Input data must match one of the models: {list(cls.class_list.keys())}"
            f"Errors occurred:\n{exception_messages}"
        )



from __future__ import annotations
from typing import Union
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .payment import Payment
from .v1paymentssubscriptions_subscription_items import (
    V1paymentssubscriptionsSubscriptionItems,
)
from .payment_method_type import PaymentMethodType


class PaymentsSubscriptionsBodyPaymentMethodGuard(OneOfBaseModel):
    class_list = {"str": str, "PaymentMethodType": PaymentMethodType}


PaymentsSubscriptionsBodyPaymentMethod = Union[str, PaymentMethodType]


@JsonMap({})
class PaymentsSubscriptionsBody(BaseModel):
    """PaymentsSubscriptionsBody

    :param billing: Determines the method of billing. Set to **pay_automatically**., defaults to None
    :type billing: str, optional
    :param billing_cycle_anchor: Determines the start of the next billing cycle, as defined in the plan described in the 'items'. Value must be timestamp in Unix time., defaults to None
    :type billing_cycle_anchor: float, optional
    :param cancel_at_period_end: Terminates the subscription at the end of the current billing period., defaults to None
    :type cancel_at_period_end: bool, optional
    :param coupon: The ID of a discount coupon that applies to all subscription items associated with this subscription. If the coupon defines a fixed monetary discount, it must use the same currency as the subscription's plan., defaults to None
    :type coupon: str, optional
    :param customer: ID of the customer who pays for this subscription. String starting with cus_.
    :type customer: str
    :param days_until_due: Number of days from the invoice date for customer to complete the payment., defaults to None
    :type days_until_due: float, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payment_fields: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
    :type payment_fields: Payment, optional
    :param payment_method: Payment Method object or ID., defaults to None
    :type payment_method: PaymentsSubscriptionsBodyPaymentMethod, optional
    :param simultaneous_invoice: Determines whether each upcoming subscription invoice is issued immediately. When true, the invoice is issued when the subscription is created. When false (the default), the invoice is issued an hour after the subscription is created., defaults to None
    :type simultaneous_invoice: bool, optional
    :param subscription_items: Array of subscription items. Each item contains a plan (required) and a quantity
    :type subscription_items: List[V1paymentssubscriptionsSubscriptionItems]
    :param tax_percent: The percentage of tax that is applied to the entire amount of the invoice., defaults to None
    :type tax_percent: float, optional
    :param trial_end: The time of the end of the customer's free trial period. If trial_from_plan is set, leave this parameter unset., defaults to None
    :type trial_end: float, optional
    :param trial_from_plan: Determines whether a free trial period can be defined in a 'plan' object attached to the subscription. If trial_end is set, leave this parameter unset., defaults to None
    :type trial_from_plan: bool, optional
    :param trial_period_days: The number of days in the customer's free trial period., defaults to None
    :type trial_period_days: float, optional
    """

    def __init__(
        self,
        customer: str,
        subscription_items: List[V1paymentssubscriptionsSubscriptionItems],
        billing: str = None,
        billing_cycle_anchor: float = None,
        cancel_at_period_end: bool = None,
        coupon: str = None,
        days_until_due: float = None,
        metadata: dict = None,
        payment_fields: Payment = None,
        payment_method: PaymentsSubscriptionsBodyPaymentMethod = None,
        simultaneous_invoice: bool = None,
        tax_percent: float = None,
        trial_end: float = None,
        trial_from_plan: bool = None,
        trial_period_days: float = None,
    ):
        """PaymentsSubscriptionsBody

        :param billing: Determines the method of billing. Set to **pay_automatically**., defaults to None
        :type billing: str, optional
        :param billing_cycle_anchor: Determines the start of the next billing cycle, as defined in the plan described in the 'items'. Value must be timestamp in Unix time., defaults to None
        :type billing_cycle_anchor: float, optional
        :param cancel_at_period_end: Terminates the subscription at the end of the current billing period., defaults to None
        :type cancel_at_period_end: bool, optional
        :param coupon: The ID of a discount coupon that applies to all subscription items associated with this subscription. If the coupon defines a fixed monetary discount, it must use the same currency as the subscription's plan., defaults to None
        :type coupon: str, optional
        :param customer: ID of the customer who pays for this subscription. String starting with cus_.
        :type customer: str
        :param days_until_due: Number of days from the invoice date for customer to complete the payment., defaults to None
        :type days_until_due: float, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payment_fields: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
        :type payment_fields: Payment, optional
        :param payment_method: Payment Method object or ID., defaults to None
        :type payment_method: PaymentsSubscriptionsBodyPaymentMethod, optional
        :param simultaneous_invoice: Determines whether each upcoming subscription invoice is issued immediately. When true, the invoice is issued when the subscription is created. When false (the default), the invoice is issued an hour after the subscription is created., defaults to None
        :type simultaneous_invoice: bool, optional
        :param subscription_items: Array of subscription items. Each item contains a plan (required) and a quantity
        :type subscription_items: List[V1paymentssubscriptionsSubscriptionItems]
        :param tax_percent: The percentage of tax that is applied to the entire amount of the invoice., defaults to None
        :type tax_percent: float, optional
        :param trial_end: The time of the end of the customer's free trial period. If trial_from_plan is set, leave this parameter unset., defaults to None
        :type trial_end: float, optional
        :param trial_from_plan: Determines whether a free trial period can be defined in a 'plan' object attached to the subscription. If trial_end is set, leave this parameter unset., defaults to None
        :type trial_from_plan: bool, optional
        :param trial_period_days: The number of days in the customer's free trial period., defaults to None
        :type trial_period_days: float, optional
        """
        self.billing = self._define_str("billing", billing, nullable=True)
        self.billing_cycle_anchor = self._define_number(
            "billing_cycle_anchor", billing_cycle_anchor, nullable=True
        )
        self.cancel_at_period_end = cancel_at_period_end
        self.coupon = self._define_str("coupon", coupon, nullable=True)
        self.customer = customer
        self.days_until_due = self._define_number(
            "days_until_due", days_until_due, nullable=True
        )
        self.metadata = metadata
        self.payment_fields = self._define_object(payment_fields, Payment)
        self.payment_method = PaymentsSubscriptionsBodyPaymentMethodGuard.return_one_of(
            payment_method
        )
        self.simultaneous_invoice = simultaneous_invoice
        self.subscription_items = self._define_list(
            subscription_items, V1paymentssubscriptionsSubscriptionItems
        )
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )
        self.trial_end = self._define_number("trial_end", trial_end, nullable=True)
        self.trial_from_plan = trial_from_plan
        self.trial_period_days = self._define_number(
            "trial_period_days", trial_period_days, nullable=True
        )
